#!/usr/bin/env python

""" MultiQC functions to plot a linegraph """

from __future__ import print_function, division
from collections import OrderedDict
import base64
import inspect
import io
import logging
import os
import random
import re
import sys

from multiqc.utils import config, report, util_functions
logger = logging.getLogger(__name__)

try:
    # Import matplot lib but avoid default X environment
    import matplotlib
    matplotlib.use('Agg')
    import matplotlib.pyplot as plt
    logger.debug("Using matplotlib version {}".format(matplotlib.__version__))
except Exception as e:
    # MatPlotLib can break in a variety of ways. Fake an error message and continue without it if so.
    # The lack of the library will be handled when plots are attempted
    print("##### ERROR! MatPlotLib library could not be loaded!    #####", file=sys.stderr)
    print("##### Flat plots will instead be plotted as interactive #####", file=sys.stderr)
    print(e)

letters = 'abcdefghijklmnopqrstuvwxyz'

# Load the template so that we can access its configuration
# Do this lazily to mitigate import-spaghetti when running unit tests
_template_mod = None
def get_template_mod():
    global _template_mod
    if not _template_mod:
        _template_mod = config.avail_templates[config.template].load()
    return _template_mod

def plot (data, pconfig=None):
    """ Plot a line graph with X,Y data.
    :param data: 2D dict, first keys as sample names, then x:y data pairs
    :param pconfig: optional dict with config key:value pairs. See CONTRIBUTING.md
    :return: HTML and JS, ready to be inserted into the page
    """
    # Don't just use {} as the default argument as it's mutable. See:
    # http://python-guide-pt-br.readthedocs.io/en/latest/writing/gotchas/
    if pconfig is None:
        pconfig = {}

    # Allow user to overwrite any given config for this plot
    if 'id' in pconfig and pconfig['id'] and pconfig['id'] in config.custom_plot_config:
        for k, v in config.custom_plot_config[pconfig['id']].items():
            pconfig[k] = v

    # Given one dataset - turn it into a list
    if type(data) is not list:
        data = [data]

    # Validate config if linting
    if config.lint:
        # Get module name
        modname = ''
        callstack = inspect.stack()
        for n in callstack:
            if 'multiqc/modules/' in n[1] and 'base_module.py' not in n[1]:
                callpath = n[1].split('multiqc/modules/',1)[-1]
                modname = '>{}< '.format(callpath)
                break
        # Look for essential missing pconfig keys
        for k in ['id', 'title', 'ylab']:
            if k not in pconfig:
                errmsg = "LINT: {}Linegraph pconfig was missing key '{}'".format(modname, k)
                logger.error(errmsg)
                report.lint_errors.append(errmsg)
        # Check plot title format
        if not re.match( r'^[^:]*\S: \S[^:]*$', pconfig.get('title', '')):
            errmsg = "LINT: {} Linegraph title did not match format 'Module: Plot Name' (found '{}')".format(modname, pconfig.get('title', ''))
            logger.error(errmsg)
            report.lint_errors.append(errmsg)

    # Smooth dataset if requested in config
    if pconfig.get('smooth_points', None) is not None:
        sumcounts = pconfig.get('smooth_points_sumcounts', True)
        for i, d in enumerate(data):
            if type(sumcounts) is list:
                sumc = sumcounts[i]
            else:
                sumc = sumcounts
            data[i] = smooth_line_data(d, pconfig['smooth_points'], sumc)

    # Add sane plotting config defaults
    for idx, yp in enumerate(pconfig.get('yPlotLines', [])):
        pconfig['yPlotLines'][idx]["width"] = pconfig['yPlotLines'][idx].get("width", 2)

    # Add initial axis labels if defined in `data_labels` but not main config
    if pconfig.get('ylab') is None:
        try:
            pconfig['ylab'] = pconfig['data_labels'][0]['ylab']
        except (KeyError, IndexError):
            pass
    if pconfig.get('xlab') is None:
        try:
            pconfig['xlab'] = pconfig['data_labels'][0]['xlab']
        except (KeyError, IndexError):
            pass

    # Generate the data dict structure expected by HighCharts series
    plotdata = list()
    for data_index, d in enumerate(data):
        thisplotdata = list()

        for s in sorted(d.keys()):

            # Ensure any overwritting conditionals from data_labels (e.g. ymax) are taken in consideration
            series_config = pconfig.copy()
            if 'data_labels' in pconfig and type(pconfig['data_labels'][data_index]) is dict:  # if not a dict: only dataset name is provided
                series_config.update(pconfig['data_labels'][data_index])

            pairs = list()
            maxval = 0
            if 'categories' in series_config:
                pconfig['categories'] = list()
                for k in d[s].keys():
                    pconfig['categories'].append(k)
                    pairs.append(d[s][k])
                    maxval = max(maxval, d[s][k])
            else:
                for k in sorted(d[s].keys()):
                    if k is not None:
                        if 'xmax' in series_config and float(k) > float(series_config['xmax']):
                            continue
                        if 'xmin' in series_config and float(k) < float(series_config['xmin']):
                            continue
                    if d[s][k] is not None:
                        if 'ymax' in series_config and float(d[s][k]) > float(series_config['ymax']):
                            continue
                        if 'ymin' in series_config and float(d[s][k]) < float(series_config['ymin']):
                            continue
                    pairs.append([k, d[s][k]])
                    try:
                        maxval = max(maxval, d[s][k])
                    except TypeError:
                        pass
            if maxval > 0 or series_config.get('hide_empty') is not True:
                this_series = { 'name': s, 'data': pairs }
                try:
                    this_series['color'] = series_config['colors'][s]
                except:
                    pass
                thisplotdata.append(this_series)
        plotdata.append(thisplotdata)

    # Add on annotation data series
    try:
        if pconfig.get('extra_series'):
            extra_series = pconfig['extra_series']
            if type(pconfig['extra_series']) == dict:
                extra_series = [[ pconfig['extra_series'] ]]
            elif type(pconfig['extra_series']) == list and type(pconfig['extra_series'][0]) == dict:
                extra_series = [ pconfig['extra_series'] ]
            for i, es in enumerate(extra_series):
                for s in es:
                    plotdata[i].append(s)
    except (KeyError, IndexError):
        pass

    # Make a plot - template custom, or interactive or flat
    try:
        return get_template_mod().linegraph(plotdata, pconfig)
    except (AttributeError, TypeError):
        if config.plots_force_flat or (not config.plots_force_interactive and len(plotdata[0]) > config.plots_flat_numseries):
            try:
                return matplotlib_linegraph(plotdata, pconfig)
            except Exception as e:
                logger.error("############### Error making MatPlotLib figure! Falling back to HighCharts.")
                logger.debug(e, exc_info=True)
                return highcharts_linegraph(plotdata, pconfig)
        else:
            # Use MatPlotLib to generate static plots if requested
            if config.export_plots:
                matplotlib_linegraph(plotdata, pconfig)
            # Return HTML for HighCharts dynamic plot
            return highcharts_linegraph(plotdata, pconfig)



def highcharts_linegraph (plotdata, pconfig=None):
    """
    Build the HTML needed for a HighCharts line graph. Should be
    called by linegraph.plot(), which properly formats input data.
    """
    if pconfig is None:
        pconfig = {}

    # Get the plot ID
    if pconfig.get('id') is None:
        pconfig['id'] = 'mqc_hcplot_'+''.join(random.sample(letters, 10))

    # Sanitise plot ID and check for duplicates
    pconfig['id'] = report.save_htmlid(pconfig['id'])

    # Build the HTML for the page
    html = '<div class="mqc_hcplot_plotgroup">'

    # Log Switch
    if pconfig.get('logswitch') is True:
        c_active = 'active'
        l_active = ''
        if pconfig.get('logswitch_active') is True:
            c_active = ''
            l_active = 'active'
        c_label = pconfig.get('cpswitch_counts_label', 'Counts')
        l_label = pconfig.get('logswitch_label', 'Log10')
        html += '<div class="btn-group hc_switch_group"> \n'
        html += '<button class="btn btn-default btn-sm {c_a}" data-action="set_numbers" data-target="{id}" data-ylab="{c_l}">{c_l}</button> \n'.format(id=pconfig['id'], c_a=c_active, c_l=c_label)
        if pconfig.get('logswitch') is True:
            html += '<button class="btn btn-default btn-sm {l_a}" data-action="set_log" data-target="{id}" data-ylab="{l_l}">{l_l}</button> \n'.format(id=pconfig['id'], l_a=l_active, l_l=l_label)
        html += '</div> '
        if len(plotdata) > 1:
            html += ' &nbsp; &nbsp; '

    # Buttons to cycle through different datasets
    if len(plotdata) > 1:
        html += '<div class="btn-group hc_switch_group">\n'
        for k, p in enumerate(plotdata):
            active = 'active' if k == 0 else ''
            try:
                name = pconfig['data_labels'][k]['name']
            except:
                name = k+1
            try:
                ylab = 'data-ylab="{}"'.format(pconfig['data_labels'][k]['ylab'])
            except:
                ylab = 'data-ylab="{}"'.format(name) if name != k+1 else ''
            try:
                ymax = 'data-ymax="{}"'.format(pconfig['data_labels'][k]['ymax'])
            except:
                ymax = ''
            try:
                xlab = 'data-xlab="{}"'.format(pconfig['data_labels'][k]['xlab'])
            except:
                xlab = ''
            html += '<button class="btn btn-default btn-sm {a}" data-action="set_data" {y} {ym} {x} data-newdata="{k}" data-target="{id}">{n}</button>\n'.format(a=active, id=pconfig['id'], n=name, y=ylab, ym=ymax, x=xlab, k=k)
        html += '</div>\n\n'

    # The plot div
    html += '<div class="hc-plot-wrapper"><div id="{id}" class="hc-plot not_rendered hc-line-plot"><small>loading..</small></div></div></div> \n'.format(id=pconfig['id'])

    report.num_hc_plots += 1

    report.plot_data[pconfig['id']] = {
        'plot_type': "xy_line",
        'datasets': plotdata,
        'config': pconfig
    }

    return html


def matplotlib_linegraph (plotdata, pconfig=None):
    """
    Plot a line graph with Matplot lib and return a HTML string. Either embeds a base64
    encoded image within HTML or writes the plot and links to it. Should be called by
    plot_bargraph, which properly formats the input data.
    """
    if pconfig is None:
        pconfig = {}

    # Plot group ID
    if pconfig.get('id') is None:
        pconfig['id'] = 'mqc_mplplot_'+''.join(random.sample(letters, 10))

    # Sanitise plot ID and check for duplicates
    pconfig['id'] = report.save_htmlid(pconfig['id'])

    # Individual plot IDs
    pids = []
    for k in range(len(plotdata)):
        try:
            name = pconfig['data_labels'][k]['name']
        except:
            name = k+1
        pid = 'mqc_{}_{}'.format(pconfig['id'], name)
        pid = report.save_htmlid(pid, skiplint=True)
        pids.append(pid)

    html = '<p class="text-info"><small><span class="glyphicon glyphicon-picture" aria-hidden="true"></span> ' + \
          'Flat image plot. Toolbox functions such as highlighting / hiding samples will not work ' + \
          '(see the <a href="http://multiqc.info/docs/#flat--interactive-plots" target="_blank">docs</a>).</small></p>'
    html += '<div class="mqc_mplplot_plotgroup" id="{}">'.format(pconfig['id'])

    # Same defaults as HighCharts for consistency
    default_colors = ['#7cb5ec', '#434348', '#90ed7d', '#f7a35c', '#8085e9',
                      '#f15c80', '#e4d354', '#2b908f', '#f45b5b', '#91e8e1']

    # Buttons to cycle through different datasets
    if len(plotdata) > 1 and not config.simple_output:
        html += '<div class="btn-group mpl_switch_group mqc_mplplot_bargraph_switchds">\n'
        for k, p in enumerate(plotdata):
            pid = pids[k]
            active = 'active' if k == 0 else ''
            try:
                name = pconfig['data_labels'][k]['name']
            except:
                name = k+1
            html += '<button class="btn btn-default btn-sm {a}" data-target="#{pid}">{n}</button>\n'.format(a=active, pid=pid, n=name)
        html += '</div>\n\n'

    # Go through datasets creating plots
    for pidx, pdata in enumerate(plotdata):

        # Plot ID
        pid = pids[pidx]

        # Save plot data to file
        fdata = OrderedDict()
        lastcats = None
        sharedcats = True
        for d in pdata:
            fdata[d['name']] = OrderedDict()
            for i, x in enumerate(d['data']):
                if type(x) is list:
                    fdata[d['name']][str(x[0])] = x[1]
                    # Check to see if all categories are the same
                    if lastcats is None:
                        lastcats = [x[0] for x in d['data']]
                    elif lastcats != [x[0] for x in d['data']]:
                        sharedcats = False
                else:
                    try:
                        fdata[d['name']][pconfig['categories'][i]] = x
                    except (KeyError, IndexError):
                        fdata[d['name']][str(i)] = x

        # Custom tsv output if the x axis varies
        if not sharedcats and config.data_format == 'tsv':
            fout = ''
            for d in pdata:
                fout += "\t"+"\t".join([str(x[0]) for x in d['data']])
                fout += "\n{}\t".format(d['name'])
                fout += "\t".join([str(x[1]) for x in d['data']])
                fout += "\n"
            with io.open (os.path.join(config.data_dir, '{}.txt'.format(pid)), 'w', encoding='utf-8') as f:
                print( fout.encode('utf-8', 'ignore').decode('utf-8'), file=f )
        else:
            util_functions.write_data_file(fdata, pid)

        # Set up figure
        fig = plt.figure(figsize=(14, 6), frameon=False)
        axes = fig.add_subplot(111)

        # Go through data series
        for idx, d in enumerate(pdata):

            # Default colour index
            cidx = idx
            while cidx >= len(default_colors):
                cidx -= len(default_colors)

            # Line style
            linestyle = 'solid'
            if d.get('dashStyle', None) == 'Dash':
                linestyle = 'dashed'

            # Reformat data (again)
            try:
                axes.plot([x[0] for x in d['data']], [x[1] for x in d['data']], label=d['name'], color=d.get('color', default_colors[cidx]), linestyle=linestyle, linewidth=1, marker=None)
            except TypeError:
                # Categorical data on x axis
                axes.plot(d['data'], label=d['name'], color=d.get('color', default_colors[cidx]), linewidth=1, marker=None)

        # Tidy up axes
        axes.tick_params(labelsize=8, direction='out', left=False, right=False, top=False, bottom=False)
        axes.set_xlabel(pconfig.get('xlab', ''))
        axes.set_ylabel(pconfig.get('ylab', ''))

        # Dataset specific y label
        try:
            axes.set_ylabel(pconfig['data_labels'][pidx]['ylab'])
        except:
            pass

        # Axis limits
        default_ylimits = axes.get_ylim()
        ymin = default_ylimits[0]
        if 'ymin' in pconfig:
            ymin = pconfig['ymin']
        elif 'yFloor' in pconfig:
            ymin = max(pconfig['yFloor'], default_ylimits[0])
        ymax = default_ylimits[1]
        if 'ymax' in pconfig:
            ymax = pconfig['ymax']
        elif 'yCeiling' in pconfig:
            ymax = min(pconfig['yCeiling'], default_ylimits[1])
        if (ymax - ymin) < pconfig.get('yMinRange', 0):
            ymax = ymin + pconfig['yMinRange']
        axes.set_ylim((ymin, ymax))

        # Dataset specific ymax
        try:
            axes.set_ylim((ymin, pconfig['data_labels'][pidx]['ymax']))
        except:
            pass

        default_xlimits = axes.get_xlim()
        xmin = default_xlimits[0]
        if 'xmin' in pconfig:
            xmin = pconfig['xmin']
        elif 'xFloor' in pconfig:
            xmin = max(pconfig['xFloor'], default_xlimits[0])
        xmax = default_xlimits[1]
        if 'xmax' in pconfig:
            xmax = pconfig['xmax']
        elif 'xCeiling' in pconfig:
            xmax = min(pconfig['xCeiling'], default_xlimits[1])
        if (xmax - xmin) < pconfig.get('xMinRange', 0):
            xmax = xmin + pconfig['xMinRange']
        axes.set_xlim((xmin, xmax))

        # Plot title
        if 'title' in pconfig:
            plt.text(0.5, 1.05, pconfig['title'], horizontalalignment='center', fontsize=16, transform=axes.transAxes)
        axes.grid(True, zorder=10, which='both', axis='y', linestyle='-', color='#dedede', linewidth=1)

        # X axis categories, if specified
        if 'categories' in pconfig:
            axes.set_xticks([i for i,v in enumerate(pconfig['categories'])])
            axes.set_xticklabels(pconfig['categories'])

        # Axis lines
        xlim = axes.get_xlim()
        axes.plot([xlim[0], xlim[1]], [0, 0], linestyle='-', color='#dedede', linewidth=2)
        axes.set_axisbelow(True)
        axes.spines['right'].set_visible(False)
        axes.spines['top'].set_visible(False)
        axes.spines['bottom'].set_visible(False)
        axes.spines['left'].set_visible(False)

        # Background colours, if specified
        if 'yPlotBands' in pconfig:
            xlim = axes.get_xlim()
            for pb in pconfig['yPlotBands']:
                axes.barh(pb['from'], xlim[1], height = pb['to']-pb['from'], left=xlim[0], color=pb['color'], linewidth=0, zorder=0, align='edge')
        if 'xPlotBands' in pconfig:
            ylim = axes.get_ylim()
            for pb in pconfig['xPlotBands']:
                axes.bar(pb['from'], ylim[1], width = pb['to']-pb['from'], bottom=ylim[0], color=pb['color'], linewidth=0, zorder=0, align='edge')

        # Tight layout - makes sure that legend fits in and stuff
        if len(pdata) <= 15:
            axes.legend(loc='lower center', bbox_to_anchor=(0, -0.22, 1, .102), ncol=5, mode='expand', fontsize=8, frameon=False)
            plt.tight_layout(rect=[0,0.08,1,0.92])
        else:
            plt.tight_layout(rect=[0,0,1,0.92])

        # Should this plot be hidden on report load?
        hidediv = ''
        if pidx > 0:
            hidediv = ' style="display:none;"'

        # Save the plot to the data directory if export is requests
        if config.export_plots:
            for fformat in config.export_plot_formats:
                # Make the directory if it doesn't already exist
                plot_dir = os.path.join(config.plots_dir, fformat)
                if not os.path.exists(plot_dir):
                    os.makedirs(plot_dir)
                # Save the plot
                plot_fn = os.path.join(plot_dir, '{}.{}'.format(pid, fformat))
                fig.savefig(plot_fn, format=fformat, bbox_inches='tight')

        # Output the figure to a base64 encoded string
        if getattr(get_template_mod(), 'base64_plots', True) is True:
            img_buffer = io.BytesIO()
            fig.savefig(img_buffer, format='png', bbox_inches='tight')
            b64_img = base64.b64encode(img_buffer.getvalue()).decode('utf8')
            img_buffer.close()
            html += '<div class="mqc_mplplot" id="{}"{}><img src="data:image/png;base64,{}" /></div>'.format(pid, hidediv, b64_img)

        # Save to a file and link <img>
        else:
            plot_relpath = os.path.join(config.plots_dir_name, 'png', '{}.png'.format(pid))
            html += '<div class="mqc_mplplot" id="{}"{}><img src="{}" /></div>'.format(pid, hidediv, plot_relpath)

        plt.close(fig)


    # Close wrapping div
    html += '</div>'

    report.num_mpl_plots += 1

    return html


def smooth_line_data(data, numpoints, sumcounts=True):
    """
    Function to take an x-y dataset and use binning to smooth to a maximum number of datapoints.
    Each datapoint in a smoothed dataset corresponds to the first point in a bin.

    Examples to show the idea:

    d=[0 1 2 3 4 5 6 7 8 9], numpoints=6
    we want to keep the first and the last element, thus excluding the last element from the binning:
    binsize = len([0 1 2 3 4 5 6 7 8]))/(numpoints-1) = 9/5 = 1.8
    taking points in indices rounded from multiples of 1.8: [0, 1.8, 3.6, 5.4, 7.2, 9],
    ...which evaluates to first_element_in_bin_indices=[0, 2, 4, 5, 7, 9]
    picking up the elements: [0 _ 2 _ 4 5 _ 7 _ 9]

    d=[0 1 2 3 4 5 6 7 8 9], numpoints=9
    binsize = 9/8 = 1.125
    indices: [0.0, 1.125, 2.25, 3.375, 4.5, 5.625, 6.75, 7.875, 9] -> [0, 1, 2, 3, 5, 6, 7, 8, 9]
    picking up the elements: [0 1 2 3 _ 5 6 7 8 9]

    d=[0 1 2 3 4 5 6 7 8 9], numpoints=3
    binsize = len(d)/numpoints = 9/2 = 4.5
    incides: [0.0, 4.5, 9] -> [0, 5, 9]
    picking up the elements: [0 _ _ _ _ 5 _ _ _ 9]
    """
    smoothed_data = dict()
    for s_name, d in data.items():
        # Check that we need to smooth this data
        if len(d) <= numpoints or len(d) == 0:
            smoothed_data[s_name] = d
            continue

        binsize = (len(d) - 1) / (numpoints - 1)
        first_element_indices = [round(binsize * i) for i in range(numpoints)]
        smoothed_d = OrderedDict(xy for i, xy in enumerate(d.items()) if i in first_element_indices)
        smoothed_data[s_name] = smoothed_d

    return smoothed_data



#!/usr/bin/env python

""" MultiQC module to parse output from Kallisto """

from __future__ import print_function
from collections import OrderedDict
import os
import logging
import re

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """ Kallisto module """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(
            name = 'Kallisto',
            anchor = 'kallisto',
            href = "http://pachterlab.github.io/kallisto/",
            info = "is a program for quantifying abundances of transcripts from RNA-Seq data."
        )

        # Find and load any Kallisto reports
        self.kallisto_data = dict()
        for f in self.find_log_files('kallisto', filehandles=True):
            self.parse_kallisto_log(f)

        # Filter to strip out ignored sample names
        self.kallisto_data = self.ignore_samples(self.kallisto_data)

        if len(self.kallisto_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.kallisto_data)))

        # Write parsed report data to a file
        self.write_data_file(self.kallisto_data, 'multiqc_kallisto')

        # Basic Stats Table
        self.kallisto_general_stats_table()

        # Alignment Rate Plot
        self.add_section( plot = self.kallisto_alignment_plot() )


    def parse_kallisto_log(self, f):
        s_name = total_reads = paligned_reads = fraglength = None
        for l in f['f']:

            # Get input filename
            match = re.search(r'\[quant\] will process (pair|file) 1: (\S+)', l)
            if match:
                s_name = self.clean_s_name(os.path.basename(match.group(2)), f['root'])

            if s_name is not None:
                # Alignment rates
                aligned = re.search(r'\[quant\] processed ([\d,]+) reads, ([\d,]+) reads pseudoaligned', l)
                if aligned:
                    total_reads = float(aligned.group(1).replace(',',''))
                    paligned_reads = float(aligned.group(2).replace(',',''))

                # Paired end fragment lengths
                flength = re.search(r'\[quant\] estimated average fragment length: ([\d\.]+)', l)
                if flength:
                    fraglength = float(flength.group(1).replace(',',''))

                if 'quantifying the abundances' in l:
                    if s_name in self.kallisto_data:
                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                    self.add_data_source(f, s_name)
                    self.kallisto_data[s_name] = {
                        'total_reads': total_reads,
                        'pseudoaligned_reads': paligned_reads,
                        'not_pseudoaligned_reads': total_reads - paligned_reads,
                        'percent_aligned': (paligned_reads / total_reads) * 100
                    }
                    if fraglength is not None:
                        self.kallisto_data[s_name]['fragment_length'] = fraglength
                    s_name = total_reads = paligned_reads = fraglength = None

    def kallisto_general_stats_table(self):
        """ Take the parsed stats from the Kallisto report and add it to the
        basic stats table at the top of the report """

        headers = OrderedDict()
        headers['fragment_length'] = {
            'title': 'Frag Length',
            'description': 'Estimated average fragment length',
            'min': 0,
            'suffix': 'bp',
            'scale': 'RdYlGn'
        }
        headers['percent_aligned'] = {
            'title': '% Aligned',
            'description': '% processed reads that were pseudoaligned',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'YlGn'
        }
        headers['pseudoaligned_reads'] = {
            'title': '{} Aligned'.format(config.read_count_prefix),
            'description': 'Pseudoaligned reads ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.kallisto_data, headers)

    def kallisto_alignment_plot (self):
        """ Make the HighCharts HTML to plot the alignment rates """

        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['pseudoaligned_reads'] = { 'color': '#437bb1', 'name': 'Pseudoaligned' }
        keys['not_pseudoaligned_reads'] =   { 'color': '#b1084c', 'name': 'Not aligned' }

        # Config for the plot
        config = {
            'id': 'kallisto_alignment',
            'title': 'Kallisto: Alignment Scores',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads'
        }

        return bargraph.plot(self.kallisto_data, keys, config)



#!/usr/bin/env python
from __future__ import print_function

import re
from collections import defaultdict
from multiqc.modules.base_module import BaseMultiqcModule
from multiqc.modules.qualimap.QM_BamQC import coverage_histogram_helptext, genome_fraction_helptext
from multiqc.plots import linegraph

# Initialise the logger
import logging
log = logging.getLogger(__name__)


class DragenCoverageHist(BaseMultiqcModule):
    def add_coverage_hist(self):
        data_by_phenotype_by_sample = defaultdict(dict)

        for f in self.find_log_files('dragen/wgs_fine_hist'):
            data_by_phenotype = parse_wgs_fine_hist(f)
            if f['s_name'] in data_by_phenotype_by_sample:
                log.debug('Duplicate sample name found! Overwriting: {}'.format(f['s_name']))
            self.add_data_source(f, section='stats')
            data_by_phenotype_by_sample[f['s_name']].update(data_by_phenotype)

        # Filter to strip out ignored sample names:
        data_by_phenotype_by_sample = self.ignore_samples(data_by_phenotype_by_sample)

        # Merge tumor and normal data:
        data_by_sample = defaultdict(dict)
        for sn in data_by_phenotype_by_sample:
            for phenotype in data_by_phenotype_by_sample[sn]:
                new_sn = sn
                if phenotype == 'normal':
                    new_sn = sn + '_normal'
                data_by_sample[new_sn] = data_by_phenotype_by_sample[sn][phenotype]

        if not data_by_sample:
            return set()

        dist_data = {sn: dist for sn, (dist, cum, depth_1pc) in data_by_sample.items()}
        cum_data  = {sn: cum  for sn, (dist, cum, depth_1pc) in data_by_sample.items()}
        depth_1pc = max(depth_1pc for (dist, cum, depth_1pc) in data_by_sample.values())

        self.add_section(
            name='Coverage distribution',
            anchor='dragen-coverage-distribution',
            description='Number of locations in the reference genome with a given depth of coverage.',
            helptext=coverage_histogram_helptext,
            plot=linegraph.plot(dist_data, {
                'id': 'dragen_coverage_dist',
                'title': 'Dragen: Coverage distribution',
                'xlab': 'Depth (x)',
                'ylab': 'Number of bases in genome covered by X reads',
                'ymin': 0,
                'xmin': 0,
                'xmax': depth_1pc,  # trim long flat tail
                'tt_label': '<b>{point.x}X</b>: {point.y} loci',
                'cpswitch': True,
            })
        )

        self.add_section(
            name='Cumulative coverage hist',
            anchor='dragen-cum-coverage-histogram',
            description='Number of locations in the reference genome with at least given depth of coverage.',
            helptext=genome_fraction_helptext,
            plot=linegraph.plot(cum_data, {
                'id': 'dragen_cumulative_coverage_hist',
                'title': 'Dragen: Cumulative coverage hist',
                'xlab': 'Depth (x)',
                'ylab': '% of bases in genome covered by at least X reads',
                'ymin': 0,
                'ymax': 100,
                'xmin': 0,
                'xmax': depth_1pc,  # trim long flat tail
                'tt_label': '<b>{point.x}X</b>: {point.y:.2f}%',
            })
        )
        return data_by_sample.keys()


def parse_wgs_fine_hist(f):
    """
    T_SRR7890936_50pc.wgs_fine_hist_normal.csv
    T_SRR7890936_50pc.wgs_fine_hist_tumor.csv

    Depth,Overall
    0,104231614
    1,9430586
    2,5546235
    ...
    998,208
    999,177
    1000+,201801

    Contains two columns: Depth and Overall. The value in the Depth column ranges from 0 to 1000+
    and the Overall column indicates the number of loci covered at the corresponding depth.

    Parsing all values except for 1000+ and plotting a distribution histogram and cumulative histogram
    """

    # first pass to calculate total number of bases to calculate percentages
    parsed_data = dict()
    for line in f['f'].splitlines():
        if line.startswith('Depth,Overall'):
            continue
        key, cnt = line.split(',')
        try:
            cnt = int(cnt)
        except ValueError:
            continue
        parsed_data[key] = cnt

    total_cnt = sum(parsed_data.values())

    data = dict()
    cum_data = dict()
    cum_cnt = 0
    depth_1pc = None

    for key, cnt in reversed(list(parsed_data.items())):
        try:
            depth = int(key)
        except ValueError:
            continue
        cum_cnt += cnt
        cum_pct = cum_cnt / total_cnt * 100.0
        if cum_pct < 1:  # to trim long flat tail
            depth_1pc = depth
        data[depth] = cnt
        cum_data[depth] = cum_pct

    m = re.search(r'(.*).wgs_fine_hist_?(\S*)?.csv', f['fn'])
    sample, phenotype = m.group(1), m.group(2)
    f['s_name'] = sample
    return {phenotype: (data, cum_data, depth_1pc)}



#!/usr/bin/env python

""" MultiQC module to parse output from Afterqc """

from __future__ import print_function
from collections import OrderedDict
import logging
import json

from multiqc import config
from multiqc.plots import bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    AfterQC module class
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='AfterQC', anchor='afterqc',
        href='https://github.com/OpenGene/AfterQC',
        info="Automatic Filtering, Trimming, Error Removing and Quality Control for fastq data.")

        # Find and load any Afterqc reports
        self.afterqc_data = dict()
        for f in self.find_log_files('afterqc', filehandles=True):
            self.parse_afterqc_log(f)

        # Filter to strip out ignored sample names
        self.afterqc_data = self.ignore_samples(self.afterqc_data)

        if len(self.afterqc_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.afterqc_data)))

        # Write parsed report data to a file
        self.write_data_file(self.afterqc_data, 'multiqc_afterqc')

        # Basic Stats Table
        self.afterqc_general_stats_table()

        # Alignment bar plot
        self.add_section (
            name = 'Bad Reads',
            anchor = 'after_qc',
            description = 'Filtering statistics of sampled reads.',
            plot = self.after_qc_bad_reads_chart()
        )

    def parse_afterqc_log(self, f):
        """ Parse the JSON output from AfterQC and save the summary statistics """
        try:
            parsed_json = json.load(f['f'])
        except:
            log.warning("Could not parse AfterQC JSON: '{}'".format(f['fn']))
            return None

        # AfterQC changed the name of their summary key at some point
        if 'summary' in parsed_json:
            summaryk = 'summary'
        elif 'afterqc_main_summary' in parsed_json:
            summaryk = 'afterqc_main_summary'
        else:
            log.warning("AfterQC JSON did not have a 'summary' or 'afterqc_main_summary' key, skipping: '{}'".format(f['fn']))
            return None

        s_name = f['s_name']
        self.add_data_source(f, s_name)
        self.afterqc_data[s_name] = {}
        for k in parsed_json[summaryk]:
            try:
                self.afterqc_data[s_name][k] = float(parsed_json[summaryk][k])
            except ValueError:
                self.afterqc_data[s_name][k] = parsed_json[summaryk][k]
        try:
            self.afterqc_data[s_name]['pct_good_bases'] = (self.afterqc_data[s_name]['good_bases'] / self.afterqc_data[s_name]['total_bases']) * 100.0
        except KeyError:
            pass

    def afterqc_general_stats_table(self):
        """ Take the parsed stats from the Afterqc report and add it to the
        General Statistics table at the top of the report """

        headers = OrderedDict()
        headers['pct_good_bases'] = {
            'title': '% Good Bases',
            'description': 'Percent Good Bases',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'BuGn',
        }
        headers['good_reads'] = {
            'title': '{} Good Reads'.format(config.read_count_prefix),
            'description': 'Good Reads ({})'.format(config.read_count_desc),
            'min': 0,
            'modify': lambda x: x * config.read_count_multiplier,
            'scale': 'GnBu',
            'shared_key': 'read_count'
        }
        headers['total_reads'] = {
            'title': '{} Total Reads'.format(config.read_count_prefix),
            'description': 'Total Reads ({})'.format(config.read_count_desc),
            'min': 0,
            'modify': lambda x: x * config.read_count_multiplier,
            'scale': 'Blues',
            'shared_key': 'read_count'
        }
        headers['readlen'] = {
            'title': 'Read Length',
            'description': 'Read Length',
            'min': 0,
            'suffix': ' bp',
            'format': '{:,.0f}',
            'scale': 'YlGn'
        }
        self.general_stats_addcols(self.afterqc_data, headers)

    def after_qc_bad_reads_chart(self):
        """ Function to generate the AfterQC bad reads bar plot """
        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['good_reads'] =                     { 'name': 'Good Reads' }
        keys['bad_reads_with_bad_barcode'] =     { 'name': 'Bad Barcode' }
        keys['bad_reads_with_bad_overlap'] =     { 'name': 'Bad Overlap' }
        keys['bad_reads_with_bad_read_length'] = { 'name': 'Bad Read Length' }
        keys['bad_reads_with_low_quality'] =     { 'name': 'Low Quality' }
        keys['bad_reads_with_polyX'] =           { 'name': 'PolyX' }
        keys['bad_reads_with_reads_in_bubble'] = { 'name': 'Reads In Bubble' }
        keys['bad_reads_with_too_many_N'] =      { 'name': 'Too many N' }

        # Config for the plot
        pconfig = {
            'id': 'afterqc_bad_reads_plot',
            'title': 'AfterQC: Filtered Reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads',
            'hide_zero_cats': False,
        }
        return bargraph.plot(self.afterqc_data, keys, pconfig)



#!/usr/bin/env python

""" MultiQC module to parse output from QUAST """

from __future__ import print_function
from collections import OrderedDict
import logging
import re

from multiqc import config
from multiqc.plots import table, bargraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule):

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='QUAST', anchor='quast',
        href="http://quast.bioinf.spbau.ru/",
        info="is a quality assessment tool for genome assemblies, written by " \
             "the Center for Algorithmic Biotechnology.")

        # Get modifiers from config file
        qconfig = getattr(config, "quast_config", {})

        self.contig_length_multiplier = qconfig.get('contig_length_multiplier', 0.001)
        self.contig_length_suffix = qconfig.get('contig_length_suffix', 'Kbp')

        self.total_length_multiplier = qconfig.get('total_length_multiplier', 0.000001)
        self.total_length_suffix = qconfig.get('total_length_suffix', 'Mbp')

        self.total_number_contigs_multiplier = qconfig.get('total_number_contigs_multiplier', 0.001)
        self.total_number_contigs_suffix = qconfig.get('total_number_contigs_suffix', 'K')


        # Find and load any QUAST reports
        self.quast_data = dict()
        for f in self.find_log_files('quast'):
            self.parse_quast_log(f)

        # Filter to strip out ignored sample names
        self.quast_data = self.ignore_samples(self.quast_data)

        if len(self.quast_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.quast_data)))

        # Write parsed report data to a file
        self.write_data_file(self.quast_data, 'multiqc_quast')

        # Basic Stats Table
        self.quast_general_stats_table()

        # Quast Stats Table
        self.add_section (
            name = 'Assembly Statistics',
            anchor = 'quast-stats',
            plot = self.quast_table()
        )
        # Number of contigs plot
        self.add_section (
            name = 'Number of Contigs',
            anchor = 'quast-contigs',
            description = """This plot shows the number of contigs found for each assembly, broken
                    down by length.""",
            plot = self.quast_contigs_barplot()
        )
        # Number of genes plot
        ng_pdata = self.quast_predicted_genes_barplot()
        if ng_pdata:
            self.add_section (
                name = 'Number of Predicted Genes',
                anchor = 'quast-genes',
                description = """This plot shows the number of predicted genes found for each
                          assembly, broken down by length.""",
                plot = ng_pdata
            )
        # Number of partial genes plot
        ng_pdata = self.quast_predicted_genes_barplot(partial=True)
        if ng_pdata:
            self.add_section (
                name = 'Number of Partially Predicted Genes',
                anchor = 'quast-partial-genes',
                description = """This plot shows the number of partially predicted genes found for each
                          assembly, broken down by length.""",
                plot = ng_pdata
            )

    def parse_quast_log(self, f):
        lines = f['f'].splitlines()

        # Pull out the sample names from the first row
        s_names = lines[0].split("\t")
        # Prepend directory name(s) to sample names as configured
        s_names = [self.clean_s_name(s_name, f['root'])
                   for s_name in s_names]
        for s_name in s_names[1:]:
            if s_name in self.quast_data:
                log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
            self.add_data_source(f, s_name)
            self.quast_data[s_name] = dict()

        # Parse remaining stats for each sample
        for l in lines[1:]:
            s = l.split("\t")
            k = s[0]
            for i, v in enumerate(s[1:]):
                s_name = s_names[i+1]
                partials = re.search(r"(\d+) \+ (\d+) part", v)
                if partials:
                    whole = partials.group(1)
                    partial = partials.group(2)
                    try:
                        self.quast_data[s_name][k] = float(whole)
                        self.quast_data[s_name]["{}_partial".format(k)] = float(partial)
                    except ValueError:
                        self.quast_data[s_name][k] = whole
                        self.quast_data[s_name]["{}_partial".format(k)] = partial
                else:
                    try:
                        self.quast_data[s_name][k] = float(v)
                    except ValueError:
                        self.quast_data[s_name][k] = v

    def quast_general_stats_table(self):
        """ Take the parsed stats from the QUAST report and add some to the
        General Statistics table at the top of the report """

        headers = OrderedDict()
        headers['N50'] = {
            'title': 'N50 ({})'.format(self.contig_length_suffix),
            'description': 'N50 is the contig length such that using longer or equal length contigs produces half (50%) of the bases of the assembly (kilo base pairs)',
            'min': 0,
            'suffix': self.contig_length_suffix,
            'scale': 'RdYlGn',
            'modify': lambda x: x * self.contig_length_multiplier
        }
        headers['Total length'] = {
            'title': 'Length ({})'.format(self.total_length_suffix),
            'description': 'The total number of bases in the assembly (mega base pairs).',
            'min': 0,
            'suffix': self.total_length_suffix,
            'scale': 'YlGn',
            'modify': lambda x: x * self.total_length_multiplier
        }
        self.general_stats_addcols(self.quast_data, headers)

    def quast_table(self):
        """ Write some more statistics about the assemblies in a table. """
        headers = OrderedDict()

        headers['N50'] = {
            'title': 'N50 ({})'.format(self.contig_length_suffix),
            'description': 'N50 is the contig length such that using longer or equal length contigs produces half (50%) of the bases of the assembly.',
            'min': 0,
            'suffix': self.contig_length_suffix,
            'scale': 'RdYlGn',
            'modify': lambda x: x * self.contig_length_multiplier
        }

        headers['N75'] = {
            'title': 'N75 ({})'.format(self.contig_length_suffix),
            'description': 'N75 is the contig length such that using longer or equal length contigs produces 75% of the bases of the assembly',
            'min': 0,
            'suffix': self.contig_length_suffix,
            'scale': 'RdYlGn',
            'modify': lambda x: x * self.contig_length_multiplier
        }

        headers['L50'] = {
            'title': 'L50 ({})'.format(self.total_number_contigs_suffix) if self.total_number_contigs_suffix else 'L50',
            'description': 'L50 is the number of contigs larger than N50, i.e. the minimum number of contigs comprising 50% of the total assembly length.',
            'min': 0,
            'suffix': self.total_number_contigs_suffix,
            'scale': 'GnYlRd',
            'modify': lambda x: x * self.total_number_contigs_multiplier
        }

        headers['L75'] = {
            'title': 'L75 ({})'.format(self.total_number_contigs_suffix) if self.total_number_contigs_suffix else 'L75',
            'description': 'L75 is the number of contigs larger than N75, i.e. the minimum number of contigs comprising 75% of the total assembly length.',
            'min': 0,
            'suffix': self.total_number_contigs_suffix,
            'scale': 'GnYlRd',
            'mofidy': lambda x: x * self.total_number_contigs_multiplier
        }
        headers['Largest contig'] = {
            'title': 'Largest contig ({})'.format(self.contig_length_suffix),
            'description': 'The size of the largest contig of the assembly',
            'min': 0,
            'suffix': self.contig_length_suffix,
            'scale': 'YlGn',
            'modify': lambda x: x * self.contig_length_multiplier
        }

        headers['Total length'] = {
            'title': 'Length ({})'.format(self.total_length_suffix),
            'description': 'The total number of bases in the assembly.',
            'min': 0,
            'suffix': self.total_length_suffix,
            'scale': 'YlGn',
            'modify': lambda x: x * self.total_length_multiplier
        }

        headers['# misassemblies'] = {
            'title': 'Misassemblies',
            'description': 'The number of positions in the assembled contigs where the left flanking sequence aligns over 1 kbp away from the right flanking sequence on the reference (relocation) or they overlap on more than 1 kbp (relocation) or flanking sequences align on different strands (inversion) or different chromosomes (translocation).',
            'scale': 'RdYlGn-rev',
            'format': '{,:.0f}'
        }
        headers['# mismatches per 100 kbp'] = {
            'title': 'Mismatches/100kbp',
            'description': 'The number of mismatches per 100 kbp',
            'scale': 'YlOrRd',
            'format': '{:,.2f}',
        }
        headers['# indels per 100 kbp'] = {
            'title': 'Indels/100kbp',
            'description': 'The number of indels per 100 kbp',
            'scale': 'YlOrRd',
            'format': '{:,.2f}',
        }
        headers['# genes'] = {
            'title': 'Genes',
            'description': '# Genes',
            'scale': 'YlGnBu',
            'format': '{:,.0f}',
            'shared_key': 'gene_count'
        }
        headers['# genes_partial'] = {
            'title': 'Genes (Partial)',
            'description': '# Genes (Partial)',
            'scale': 'YlGnBu',
            'format': '{:,.0f}',
            'shared_key': 'gene_count'
        }
        headers['# predicted genes (unique)'] = {
            'title': 'Genes',
            'description': '# Predicted Genes (Unique)',
            'scale': 'YlGnBu',
            'format': '{:,.0f}',
            'shared_key': 'gene_count'
        }
        headers['Genome fraction (%)'] = {
            'title': 'Genome Fraction',
            'description': 'The total number of aligned bases in the reference, divided by the genome size.',
            'max': 100,
            'suffix': '%',
            'scale': 'YlGn'
        }
        config = {
            'id': 'quast_table',
            'namespace': 'QUAST',
            'min': 0,
        }
        return table.plot(self.quast_data, headers, config)

    def quast_contigs_barplot(self):
        """ Make a bar plot showing the number and length of contigs for each assembly """

        # Prep the data
        data = dict()
        categories = []
        for s_name, d in self.quast_data.items():
            nums_by_t = dict()
            for k, v in d.items():
                m = re.match(r'# contigs \(>= (\d+) bp\)', k)
                if m and v != '-':
                    nums_by_t[int(m.groups()[0])] = int(v)

            tresholds = sorted(nums_by_t.keys(), reverse=True)
            p = dict()
            cats = []
            for i, t in enumerate(tresholds):
                if i == 0:
                    c = '>= ' + str(t) + ' bp'
                    cats.append(c)
                    p[c] = nums_by_t[t]
                else:
                    c = str(t) + '-' + str(tresholds[i - 1]) + ' bp'
                    cats.append(c)
                    p[c] = nums_by_t[t] - nums_by_t[tresholds[i - 1]]
            if not categories:
                categories = cats
            data[s_name] = p

        pconfig = {
            'id': 'quast_num_contigs',
            'title': 'QUAST: Number of Contigs',
            'ylab': '# Contigs',
            'yDecimals': False
        }

        return bargraph.plot(data, categories, pconfig)

    def quast_predicted_genes_barplot(self, partial=False):
        """
        Make a bar plot showing the number and length of predicted genes
        for each assembly
        """

        # Prep the data
        # extract the ranges given to quast with "--gene-thresholds"
        # keys look like:
        #   `# predicted genes (>= 300 bp)`
        #   `# predicted genes (>= 300 bp)_partial`
        pattern = re.compile(r'# predicted genes \(>= (\d+) bp\)' + ('_partial' if partial else ''))

        data = {}
        all_categories = []
        data_key = '# predicted genes (>= {} bp)' + ('_partial' if partial else '')
        for s_name, d in self.quast_data.items():
            thresholds = []
            for k in d.keys():
                m = re.match(pattern, k)
                if m:
                    thresholds.append(int(m.groups()[0]))
            thresholds = sorted(list(set(thresholds)))
            if len(thresholds) < 2:
                continue

            highest_threshold = thresholds[-1]
            highest_cat = (highest_threshold, '>= {} bp'.format(highest_threshold))  # tuple (key-for-sorting, label)
            all_categories.append(highest_cat)
            plot_data = { highest_cat[1]: d[data_key.format(highest_threshold)] }

            # converting >=T1, >=T2,.. into 0-T1, T1-T2,..
            for low, high in zip(thresholds[:-1], thresholds[1:]):
                cat = (low, '{}-{} bp'.format(low, high))
                all_categories.append(cat)
                plot_data[cat[1]] = d[data_key.format(low)] - d[data_key.format(high)]

            try:
                assert sum(plot_data.values()) == d[data_key.format(0)]
            except AssertionError:
                raise UserWarning("Predicted gene counts didn't add up properly for \"{}\"".format(s_name))

            data[s_name] = plot_data

        all_categories = [label for k, label in sorted(list(set(all_categories)))]

        if len(all_categories) > 0:
            return bargraph.plot(data, all_categories,
                                 {'id': 'quast_' + ('partially_' if partial else '') + 'predicted_genes',
                                  'title': 'QUAST: Number of ' + ('partially ' if partial else '') + 'predicted genes',
                                  'ylab': 'Number of ' + ('partially ' if partial else '') + 'predicted genes'})
        else:
            return None



from collections import OrderedDict

from multiqc.utils import config

from .plot_basic_hist import plot_basic_hist
from .plot_aqhist import plot_aqhist
from .plot_bhist import plot_bhist
from .plot_bqhist import plot_bqhist
from .plot_covhist import plot_covhist
from .plot_idhist import plot_idhist
from .plot_ihist import plot_ihist
from .plot_indelhist import plot_indelhist
from .plot_mhist import plot_mhist
from .plot_qahist import plot_qahist
from .plot_qhist import plot_qhist

class slice2OrderedDict(object):
    def __getitem__(self, keys):
        return OrderedDict([(slice.start, slice.stop) for slice in keys])
odict = slice2OrderedDict()

section_order = [
    'stats',
    'covhist',
    'covstats',
    'bincov',
    'bqhist',
    'idhist',
    'indelhist',
    'mhist',
    'qahist',
    'qhist',
    'aqhist',
    'ehist',
    'lhist',
    'ihist',
    'gchist',
    'bhist' ,

    'rpkm',
    'statsfile_machine',
    'statsfile',
]
file_types = {
    'stats': {
        'title': 'BBDuk filtering statistics',
        'descr': 'Proportion of reads that matched adapters/contaminants.',
        'help_text': '',
        'kvrows': ['Total', 'Matched'],
        'kv_descriptions': {
            'Total': (
                'Total number of reads processed',
                {
                    'description': 'Aligned Reads ({})'.format(config.read_count_desc),
                    'shared_key': 'read_count',
                    'modify': lambda x: x * config.read_count_multiplier,
                    'scale': 'PuBu',
                    'hidden': True
                }
            ),
            'Matched': (
                'Total number of reads matching adapters/contaminants',
                {
                    'description': 'Aligned Reads ({})'.format(config.read_count_desc),
                    'shared_key': 'read_count',
                    'modify': lambda x: x * config.read_count_multiplier,
                    'scale': 'Reds',
                    'hidden': True
                }
            ),
            'Percent filtered': (
                'Proportion of reads filtered, matching adapters/contaminants',
                {
                    'max': 100,
                    'min': 0,
                    'scale': 'OrRd',
                    'suffix': '%',
                    'hidden': False
                }
            ),
        },
        'cols': odict[
            'Name':str,
            'Reads':int,
            'ReadsPct':lambda v: float(v.strip("%"))],
        'plot_func': None,  ## Plotting for 'stats' not implemented
        'plot_params': {},
    },
    'aqhist': {
        'title': 'Read quality',
        'descr': 'Histogram of average read qualities (`aqhist`). '
                 'Plot shows the number of reads at each quality score.',
        'help_text': '',
        'cols': odict[
            'Quality':int,
            'count1':int,
            'fraction1':float,
            'count2':int,
            'fraction2':float
        ],
        'plot_func': plot_aqhist,
        'plot_params': {
            'xPlotBands':
            [
                {'from': 28, 'to': 100, 'color': '#c3e6c3'},
                {'from': 20, 'to': 28, 'color': '#e6dcc3'},
                {'from': 0, 'to': 20, 'color': '#e6c3c3'},
            ],
            'yLog': True,
        }
    },
#basecov
#  - Coverage per base location.
#  - too big to interpret here
    'bhist' : {
        'title': 'Base composition',
        'descr': 'Base composition histogram by position (`bhist`). '
                 'The plot shows the percentage of `G+C`, `A+T`, and `N` bases '
                 'for each position in the reads.',
        'help_text': 'Relative composition',
        'cols': odict[
            'Pos':int, 'A':float, 'C':float, 'G':float, 'T':float, 'N':float
        ],
        'plot_func': plot_bhist,
        'plot_params': {},
    },
    'bincov': {
        'title': 'Binned coverage',
        'descr': 'Binned coverage per location, one line per X bases (`bincov`).',
        'help_text': '',
        'kvrows': ['Mean', 'STDev' ],
        'cols': odict[
            'RefName':str,
            'Cov':float,
            'Pos':int,
            'RunningPos':int
        ],
        'plot_func': plot_basic_hist,
        'plot_params': {},
        'not_implemented': ''
    },
    'bqhist': {
        'title': 'Base quality',
        'descr': 'Quality histogram designed for box plots (`bqhist`). '
                 'Refer to original source files for complete boxplot data. '
                 'Plot shows mean base quality for each read position. ',
        'help_text': '',
        'cols': odict[
            'BaseNum':int,
            'count_1':int, 'min_1':int, 'max_1':int, 'mean_1':float,
            'Q1_1':int, 'med_1':int, 'Q3_1':int, 'LW_1':int, 'RW_1':int,
            'count_2':int, 'min_2':int, 'max_2':int, 'mean_2':float,
            'Q1_2':int, 'med_2':int, 'Q3_2':int, 'LW_2':int, 'RW_2':int
        ],
        'plot_func': plot_bqhist,
        'plot_params': {}
    },
    'covhist': {
        'title': 'Coverage histogram',
        'descr': 'Histogram of number of occurrences of each coverage depth level (`covhist`). '
                 'Note that lines have been smoothed to 400 points; '
                 'higher resolution data might be available in the original data source. ',
        'help_text': '',
        'cols': odict['Coverage':int, 'numBases':int],
        'plot_func': plot_covhist,
        'plot_params': {
            'yLog': True,
        }
    },
    'covstats': {
        'title': 'Coverage stats',
        'descr': 'Per-scaffold coverage info (`covstats`).',
        'help_text': '',
        'cols': odict[
            'ID':str,
            'Avg_fold':float
        ],
        'extracols': odict[
            'Length':int,
            'Ref_GC':float,
            'Covered_percent':float,
            'Covered_bases':int,
            'Plus_reads':int,
            'Minus_reads':int,
            'Median_fold':int,
            'Read_GC':float,
            'Std_Dev':float
        ],
        'plot_func': plot_basic_hist,
        'plot_params': {},
        'not_implemented': ''
    },
    'ehist': {
        'title': 'Errors-per-read',
        'descr': 'Errors-per-read histogram (`ehist`). ',
        'help_text': '',
        'cols': odict['Errors':int, 'Count':int ],
        'plot_func': plot_basic_hist,
        'plot_params': {
            'xlab': 'Errors',
            'ylab': '# Reads'
        }
    },
    'gchist' : {
        'title': 'GC content',
        'descr': 'Read GC content histogram (`gchist`).',
        'help_text': '',
        'kvrows': ['Mean', 'Median', 'Mode', 'STDev'],
        'kv_descriptions': {
            'Mean': ('Average GC content', {}),
            'Median': ('Median GC content', {}),
            'Mode': ('The most commonly occuring value of the GC content distribution', {}),
            'STDev': ('Standard deviation of average GC content', {}),
        },
        'cols': odict['GC':float, 'Count':int ],
        'plot_func': plot_basic_hist,
        'plot_params': {
            'xlab': 'Proportion GC',
            'ylab': '# Reads'
        }
    },
    'idhist': {
        'title': 'Identity histogram',
        'descr': 'Histogram of read count versus percent base pair identity '
                 'of aligned reads (`idhist`).',
        'help_text': '',
        'kvrows': ['Mean_reads', 'Mean_bases',
                   'Median_reads', 'Median_bases',
                   'Mode_reads', 'Mode_bases',
                   'STDev_reads', 'STDev_bases' ],
        'kv_descriptions': {
            'Mean_reads': ('Average percent identity of aligned reads', {}),
            'Mean_bases': ('Average percent identity of aligned bases', {}),
            'Median_reads': ('Median percent identity of aligned reads', {}),
            'Median_bases': ('Median percent identity of aligned bases', {}),
            'Mode_reads': ('The most commonly occuring average value amongst aligned reads (i.e. the mode of the distribution)', {}),
            'Mode_bases': ('The most commonly occuring average value amongst aligned bases (i.e. the mode of the distribution)', {}),
            'STDev_reads': ('The standard deviation of the average percent identity distribution for aligned reads', {}),
            'STDev_bases': ('The standard deviation of the average percent identity distribution for aligned bases', {}),
        },
        'cols': odict[
            'Identity':float, 'Reads':int, 'Bases':int
        ],
        'plot_func': plot_idhist,
        'plot_params': {}
    },
    'ihist': {
        'title': 'Insert sizes',
        'descr': 'Histogram of computed insert sizes, for paired reads (`ihist`). '
                 'Plotted data has been cut off at 99% to prevent long tails; '
                 'Complete data available in original source files.',
        'help_text': 'The insert size is the length of the sequence between the '
                     'sequencing adapters, which for most common insert sizes is '
                     'longer than the sum of both read pairs. '
                     'In some cases, the insert size is shorter than the length '
                     'of the two read pairs combined, resulting in an insert size '
                     'shorter than the sum of the length of the reads pairs.',
        'kvrows': ['Mean', 'Median', 'STDev', 'PercentOfPairs'],
        'kv_descriptions': {
            'Mean': ('Average insert length', {}),
            'Median': ('Median insert length', {}),
            'STDev': ('Standard deviation of insert size length distribution', {}),
            'PercentOfPairs': ('', {}),
        },
        'cols': odict['InsertSize':int, 'Count':int ],
        'plot_func': plot_ihist,
        'plot_params': {}
    },
    'indelhist': {
        'title': 'Indel lengths',
        'descr': 'Indel length histogram (`indelhist`). '
                 'The plots show the number of observed insertions and deletions, '
                 'for each insertion and deletion length.',
        'help_text': '',
        'cols': odict['Length':int, 'Deletions':int, 'Insertions':int],
        'plot_func': plot_indelhist,
        'plot_params': {}
    },
    'lhist' : {
        'title': 'Read lengths',
        'descr': 'Read length histogram (`lhist`).',
        'help_text': '',
        'cols': odict['Length':int, 'Count':int ],
        'plot_func': plot_basic_hist,
        'plot_params': {
            'xlab': 'Read length (base pairs)',
            'ylab': '# Reads'
        }
    },
    'mhist': {
        'title': 'Match, substitution, deletion, and insertion rates',
        'descr': 'Histogram of match, substitution, deleletion, '
                 'and insertion rates by read location (`mhist`).',
        'help_text': '',
        'cols': odict[
            'BaseNum':int,
            'Match1':float, 'Sub1':float, 'Del1':float, 'Ins1':float, 'N1':float, 'Other1':float,
            'Match2':float, 'Sub2':float, 'Del2':float, 'Ins2':float, 'N2':float, 'Other2':float
        ],
        'plot_func': plot_mhist,
        'plot_params': {}
    },
    'qahist': {
        'title': 'Quality accuracy',
        'descr': 'Base quality accuracy histogram of error rates versus quality score (`qahist`). '
                 'The plots show the observed count of each type of alignment by base quality score.',
        'help_text': '',
        'kvrows': ['Deviation', 'DeviationSub' ],
        'kv_descriptions': {
            'Deviation': ('', {}),
            'DeviationSub': ('', {}),
        },
        'cols': odict[
            'Quality':int, 'Match':int, 'Sub':int, 'Ins':int, 'Del':int,
            'TrueQuality':float, 'TrueQualitySub':float
        ],
        'plot_func': plot_qahist,
        'plot_params': {}
    },
    'qhist': {
        'title': 'Sequence Quality Histograms',
        'descr': 'Quality histogram by position (`qhist`). '
                 'The plots show the average quality for each position in the reads, '
                 'using the linear values, logarithmically scaled values, and the '
                 'actual measured qualities based on the alignments.',
        'help_text': '',
        'cols': odict[
            'BaseNum':int,
            'Read1_linear':float,
            'Read1_log':float,
            'Read1_measured':float,
            'Read2_linear':float,
            'Read2_log':float,
            'Read2_measured':float
        ],
        'plot_func': plot_qhist,
        'plot_params': {}
    },
    'rpkm': {
        'title': 'RPKM/FPKM',
        'descr': 'Per-scaffold RPKM/FPKM counts (`rpkm`).',
        'help_text': '',
        'kvrows': ['File', 'Reads', 'Mapped', 'RefSequences' ],
        'kv_descriptions': {
            'File': ('', {}),
            'Reads': ('', {}),
            'Mapped': ('', {}),
            'RefSequences': ('', {}),
        },
        'cols': odict[
            'Name':str,
            'Length':int, 'Bases':int, 'Coverage':float,
            'Reads':int, 'RPKM':float, 'Frags':int, 'FPKM':float
        ],
        'plot_func': plot_basic_hist,
        'plot_params': {},
        'not_implemented': '',
    },
    'statsfile_machine': {
        'title': 'General stats',
        'descr': 'General Stats (`statsfile_machine`).',
        'help_text': '',
        'cols': [],
        'plot_func': plot_basic_hist,
        'plot_params': {}
    },
    'statsfile': {
        'title': 'General stats',
        'descr': 'General Stats (`statsfile`).',
        'help_text': '',
        'cols': [],
        'plot_params': {},
        'plot_func': plot_basic_hist,
        'not_implemented': ''
    }
}

statsfile_machine_keys = [
    'Reads_Used', 'Bases_Used', 'Reads/sec', 'kBases/sec',

    'R1_Mapped_Percent', 'R1_Unambiguous_Percent', 'R1_Mapped_Reads',
    'R1_Unambiguous_Reads', 'Mated_Pairs', 'Bad_Pairs', 'R1_Rescued',
    'Avg_Insert_Size', 'R1_Perfect_Best_Site', 'R1_Semiperfect_Site',
    'R1_Ambiguous_Mapping', 'R1_Low_Quality_Discards', 'R1_Match_Rate',
    'R1_Error_Rate', 'R1_Sub_Rate', 'R1_Del_Rate', 'R1_Ins_Rate',
    'R1_N_Rate', 'R1_Match_Count', 'R1_Error_Count', 'R1_Sub_Count',
    'R1_Del_Count', 'R1_Ins_Count', 'R1_N_Count',

    'R2_Mapped_Percent', 'R2_Unambiguous_Percent', 'R2_Mapped_Reads',
    'R2_Unambiguous_Reads', 'R2_Rescued', 'R2_Perfect_Best_Site',
    'R2_Semiperfect_Site', 'R2_Ambiguous_Mapping', 'R2_Low_Quality_Discards',
    'R2_Match_Rate', 'R2_Error_Rate', 'R2_Sub_Rate', 'R2_Del_Rate',
    'R2_Ins_Rate', 'R2_N_Rate', 'R2_Match_Count', 'R2_Error_Count',
    'R2_Sub_Count', 'R2_Del_Count', 'R2_Ins_Count', 'R2_N_Count',
    'R2_Mapped_Percent',

]



#!/usr/bin/env python

""" MultiQC module to parse output from leeHom """

from __future__ import print_function
import logging
import re

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    leeHom module class, parses stderr logs.
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='leeHom', anchor='leehom',
        href='https://github.com/grenaud/leeHom',
        info="is a program for the Bayesian reconstruction of ancient DNA")

        # Find and load any leeHom reports
        self.leehom_data = dict()

        for f in self.find_log_files('leehom', filehandles=True):
            parsed_data = self.parse_leehom_logs(f)
            if parsed_data is not None and len(parsed_data) > 0:
                self.leehom_data[f['s_name']] = parsed_data
                self.add_data_source(f, f['s_name'])

        # Filter to strip out ignored sample names
        self.leehom_data = self.ignore_samples(self.leehom_data)

        if len(self.leehom_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.leehom_data)))

        # Write parsed report data to a file
        self.write_data_file(self.leehom_data, 'multiqc_leehom')

        # Basic Stats Table
        self.leehom_general_stats_table()


    def parse_leehom_logs(self, f):
        """ Go through log file looking for leehom output """
        regexes = {
            'total': r"Total reads[\s\:]+(\d+)",
            'merged_trimming': r"Merged \(trimming\)\s+(\d+)",
            'merged_overlap': r"Merged \(overlap\)\s+(\d+)",
            'kept': r"Kept PE/SR\s+(\d+)",
            'trimmed': r"Trimmed SR\s+(\d+)",
            'adapter_dimers_chimeras': r"Adapter dimers/chimeras\s+(\d+)",
            'failed_key': r"Failed Key\s+(\d+)"
        }
        parsed_data = dict()
        for l in f['f']:
            # Search regexes for overview stats
            for k, r in regexes.items():
                match = re.search(r, l)
                if match:
                    parsed_data[k] = int(match.group(1))
        return parsed_data


    def leehom_general_stats_table(self):
        """ Take the parsed stats from the leeHom report and add it to the
        basic stats table at the top of the report """

        headers = {}
        headers['merged_trimming'] = {
            'title': '{} Merged (Trimming)'.format(config.read_count_prefix),
            'description': 'Merged clusters from trimming ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        headers['merged_overlap'] = {
            'title': '{} Merged (Overlap)'.format(config.read_count_prefix),
            'description': 'Merged clusters from overlapping reads ({})'.format(config.read_count_desc),
            'min': 0,
            'scale': 'PuRd',
            'modify': lambda x: x * config.read_count_multiplier,
            'shared_key': 'read_count'
        }
        self.general_stats_addcols(self.leehom_data, headers)




#!/usr/bin/env python
"""
Helper functions to manipulate colours and colour scales
"""

from __future__ import print_function
import spectra
import numpy as np
import re

# Default logger will be replaced by caller
import logging
logger = logging.getLogger(__name__)


class mqc_colour_scale(object):
    """ Class to hold a colour scheme. """

    def __init__(self, name='GnBu', minval=0, maxval=100):
        """ Initialise class with a colour scale """

        self.colours = self.get_colours(name)
        self.name = name

        # Sanity checks
        minval = re.sub("[^0-9\.]", "", str(minval))
        maxval = re.sub("[^0-9\.]", "", str(maxval))
        if minval == '':
            minval = 0
        if maxval == '':
            maxval = 100
        if float(minval) == float(maxval):
            self.minval = float(minval)
            self.maxval = float(minval) + 1.0
        elif float(minval) > float(maxval):
            self.minval = float(maxval)
            self.maxval = float(minval)
        else:
            self.minval = float(minval)
            self.maxval = float(maxval)

    def get_colour(self, val, colformat='hex'):
        """ Given a value, return a colour within the colour scale """
        try:
            # When we have non-numeric values (e.g. Male/Female, Yes/No, chromosome names, etc), and a qualitive
            # scale (Set1, Set3, etc), we don't want to attempt to parse numbers, otherwise we will end up with all
            # values assigned withthe same color. But instead we will geta has from a string to hope to assign
            # a unique color for each possible enumeration value.
            if self.name in mqc_colour_scale.qualitative_scales and isinstance(val, str):
                return self.colours[hash(val) % len(self.colours)]

            # When there is only 1 color in scale, spectra.scale() will crash with DevisionByZero
            elif len(self.colours) == 1:
                return self.colours[0]

            else:
                # Sanity checks
                val = re.sub("[^0-9\.]", "", str(val))
                if val == '':
                    val = self.minval
                val = float(val)
                val = max(val, self.minval)
                val = min(val, self.maxval)

                domain_nums = list( np.linspace(self.minval, self.maxval, len(self.colours)) )
                my_scale = spectra.scale(self.colours).domain(domain_nums)

                # Weird, I know. I ported this from the original JavaScript for continuity
                # Seems to work better than adjusting brightness / saturation / luminosity
                rgb_converter = lambda x: max(0, min(1, 1+((x-1)*0.3)))
                thecolour = spectra.rgb( *[rgb_converter(v) for v in my_scale(val).rgb] )

                return thecolour.hexcode

        except:
            # Shouldn't crash all of MultiQC just for colours
            return ''


    def get_colours(self, name='GnBu'):
        """ Function to get a colour scale by name
        Input: Name of colour scale (suffix with -rev for reversed)
               Defaults to 'GnBu' if scale not found.
        Returns: List of hex colours
        """
        # ColorBrewer colours, taken from Chroma.js source code
        # https://github.com/gka/chroma.js

        ###    Copyright (c) 2002 Cynthia Brewer, Mark Harrower, and The
        ###    Pennsylvania State University.
        ###
        ###    Licensed under the Apache License, Version 2.0 (the "License");
        ###    you may not use this file except in compliance with the License.
        ###    You may obtain a copy of the License at
        ###    http://www.apache.org/licenses/LICENSE-2.0
        ###
        ###    Unless required by applicable law or agreed to in writing, software distributed
        ###    under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
        ###    CONDITIONS OF ANY KIND, either express or implied. See the License for the
        ###    specific language governing permissions and limitations under the License.

        colorbrewer_scales = {
            # sequential
            'OrRd': ['#fff7ec', '#fee8c8', '#fdd49e', '#fdbb84', '#fc8d59', '#ef6548', '#d7301f', '#b30000', '#7f0000'],
            'PuBu': ['#fff7fb', '#ece7f2', '#d0d1e6', '#a6bddb', '#74a9cf', '#3690c0', '#0570b0', '#045a8d', '#023858'],
            'BuPu': ['#f7fcfd', '#e0ecf4', '#bfd3e6', '#9ebcda', '#8c96c6', '#8c6bb1', '#88419d', '#810f7c', '#4d004b'],
            'Oranges': ['#fff5eb', '#fee6ce', '#fdd0a2', '#fdae6b', '#fd8d3c', '#f16913', '#d94801', '#a63603', '#7f2704'],
            'BuGn': ['#f7fcfd', '#e5f5f9', '#ccece6', '#99d8c9', '#66c2a4', '#41ae76', '#238b45', '#006d2c', '#00441b'],
            'YlOrBr': ['#ffffe5', '#fff7bc', '#fee391', '#fec44f', '#fe9929', '#ec7014', '#cc4c02', '#993404', '#662506'],
            'YlGn': ['#ffffe5', '#f7fcb9', '#d9f0a3', '#addd8e', '#78c679', '#41ab5d', '#238443', '#006837', '#004529'],
            'Reds': ['#fff5f0', '#fee0d2', '#fcbba1', '#fc9272', '#fb6a4a', '#ef3b2c', '#cb181d', '#a50f15', '#67000d'],
            'RdPu': ['#fff7f3', '#fde0dd', '#fcc5c0', '#fa9fb5', '#f768a1', '#dd3497', '#ae017e', '#7a0177', '#49006a'],
            'Greens': ['#f7fcf5', '#e5f5e0', '#c7e9c0', '#a1d99b', '#74c476', '#41ab5d', '#238b45', '#006d2c', '#00441b'],
            'YlGnBu': ['#ffffd9', '#edf8b1', '#c7e9b4', '#7fcdbb', '#41b6c4', '#1d91c0', '#225ea8', '#253494', '#081d58'],
            'Purples': ['#fcfbfd', '#efedf5', '#dadaeb', '#bcbddc', '#9e9ac8', '#807dba', '#6a51a3', '#54278f', '#3f007d'],
            'GnBu': ['#f7fcf0', '#e0f3db', '#ccebc5', '#a8ddb5', '#7bccc4', '#4eb3d3', '#2b8cbe', '#0868ac', '#084081'],
            'Greys': ['#ffffff', '#f0f0f0', '#d9d9d9', '#bdbdbd', '#969696', '#737373', '#525252', '#252525', '#000000'],
            'YlOrRd': ['#ffffcc', '#ffeda0', '#fed976', '#feb24c', '#fd8d3c', '#fc4e2a', '#e31a1c', '#bd0026', '#800026'],
            'PuRd': ['#f7f4f9', '#e7e1ef', '#d4b9da', '#c994c7', '#df65b0', '#e7298a', '#ce1256', '#980043', '#67001f'],
            'Blues': ['#f7fbff', '#deebf7', '#c6dbef', '#9ecae1', '#6baed6', '#4292c6', '#2171b5', '#08519c', '#08306b'],
            'PuBuGn': ['#fff7fb', '#ece2f0', '#d0d1e6', '#a6bddb', '#67a9cf', '#3690c0', '#02818a', '#016c59', '#014636'],

            # diverging
            'Spectral': ['#9e0142', '#d53e4f', '#f46d43', '#fdae61', '#fee08b', '#ffffbf', '#e6f598', '#abdda4', '#66c2a5', '#3288bd', '#5e4fa2'],
            'RdYlGn': ['#a50026', '#d73027', '#f46d43', '#fdae61', '#fee08b', '#ffffbf', '#d9ef8b', '#a6d96a', '#66bd63', '#1a9850', '#006837'],
            'RdBu': ['#67001f', '#b2182b', '#d6604d', '#f4a582', '#fddbc7', '#f7f7f7', '#d1e5f0', '#92c5de', '#4393c3', '#2166ac', '#053061'],
            'PiYG': ['#8e0152', '#c51b7d', '#de77ae', '#f1b6da', '#fde0ef', '#f7f7f7', '#e6f5d0', '#b8e186', '#7fbc41', '#4d9221', '#276419'],
            'PRGn': ['#40004b', '#762a83', '#9970ab', '#c2a5cf', '#e7d4e8', '#f7f7f7', '#d9f0d3', '#a6dba0', '#5aae61', '#1b7837', '#00441b'],
            'RdYlBu': ['#a50026', '#d73027', '#f46d43', '#fdae61', '#fee090', '#ffffbf', '#e0f3f8', '#abd9e9', '#74add1', '#4575b4', '#313695'],
            'BrBG': ['#543005', '#8c510a', '#bf812d', '#dfc27d', '#f6e8c3', '#f5f5f5', '#c7eae5', '#80cdc1', '#35978f', '#01665e', '#003c30'],
            'RdGy': ['#67001f', '#b2182b', '#d6604d', '#f4a582', '#fddbc7', '#ffffff', '#e0e0e0', '#bababa', '#878787', '#4d4d4d', '#1a1a1a'],
            'PuOr': ['#7f3b08', '#b35806', '#e08214', '#fdb863', '#fee0b6', '#f7f7f7', '#d8daeb', '#b2abd2', '#8073ac', '#542788', '#2d004b'],

            # qualitative
            'Set2': ['#66c2a5', '#fc8d62', '#8da0cb', '#e78ac3', '#a6d854', '#ffd92f', '#e5c494', '#b3b3b3'],
            'Accent': ['#7fc97f', '#beaed4', '#fdc086', '#ffff99', '#386cb0', '#f0027f', '#bf5b17', '#666666'],
            'Set1': ['#e41a1c', '#377eb8', '#4daf4a', '#984ea3', '#ff7f00', '#ffff33', '#a65628', '#f781bf', '#999999'],
            'Set3': ['#8dd3c7', '#ffffb3', '#bebada', '#fb8072', '#80b1d3', '#fdb462', '#b3de69', '#fccde5', '#d9d9d9', '#bc80bd', '#ccebc5', '#ffed6f'],
            'Dark2': ['#1b9e77', '#d95f02', '#7570b3', '#e7298a', '#66a61e', '#e6ab02', '#a6761d', '#666666'],
            'Paired': ['#a6cee3', '#1f78b4', '#b2df8a', '#33a02c', '#fb9a99', '#e31a1c', '#fdbf6f', '#ff7f00', '#cab2d6', '#6a3d9a', '#ffff99', '#b15928'],
            'Pastel2': ['#b3e2cd', '#fdcdac', '#cbd5e8', '#f4cae4', '#e6f5c9', '#fff2ae', '#f1e2cc', '#cccccc'],
            'Pastel1': ['#fbb4ae', '#b3cde3', '#ccebc5', '#decbe4', '#fed9a6', '#ffffcc', '#e5d8bd', '#fddaec', '#f2f2f2'],
        }

        if name.startswith('#'):
            return [name]

        if name in mqc_colour_scale.html_colors:
            return [name]

        # Detect reverse colour scales
        reverse = False
        if str(name).endswith('-rev'):
            reverse = True
            name = name[:-4]

        # Default colour scale
        if name not in colorbrewer_scales:
            name = 'GnBu'

        # Return colours
        if reverse:
            return list(reversed(colorbrewer_scales[name]))
        else:
            return colorbrewer_scales[name]

    qualitative_scales = ['Set2', 'Accent', 'Set1', 'Set3', 'Dark2', 'Paired', 'Pastel2', 'Pastel1']

    html_colors = {
        "black": "#000000",
        "silver": "#C0C0C0",
        "gray": "#808080",
        "grey": "#808080",
        "white": "#FFFFFF",
        "maroon": "#800000",
        "red": "#FF0000",
        "purple": "#800080",
        "fuchsia": "#FF00FF",
        "green": "#008000",
        "lime": "#00FF00",
        "olive": "#808000",
        "yellow": "#FFFF00",
        "navy": "#000080",
        "blue": "#0000FF",
        "teal": "#008080",
        "aqua": "#00FFFF",
        "darkblue": "#00008B",
        "mediumblue": "#0000CD",
        "darkgreen": "#006400",
        "darkcyan": "#008B8B",
        "deepskyblue": "#00BFFF",
        "darkturquoise": "#00CED1",
        "mediumspringgreen": "#00FA9A",
        "springgreen": "#00FF7F",
        "cyan": "#00FFFF",
        "midnightblue": "#191970",
        "dodgerblue": "#1E90FF",
        "lightseagreen": "#20B2AA",
        "forestgreen": "#228B22",
        "seagreen": "#2E8B57",
        "darkslategray": "#2F4F4F",
        "darkslategrey": "#2F4F4F",
        "limegreen": "#32CD32",
        "mediumseagreen": "#3CB371",
        "turquoise": "#40E0D0",
        "royalblue": "#4169E1",
        "steelblue": "#4682B4",
        "darkslateblue": "#483D8B",
        "mediumturquoise": "#48D1CC",
        "indigo": "#4B0082",
        "darkolivegreen": "#556B2F",
        "cadetblue": "#5F9EA0",
        "cornflowerblue": "#6495ED",
        "rebeccapurple": "#663399",
        "mediumaquamarine": "#66CDAA",
        "dimgray": "#696969",
        "dimgrey": "#696969",
        "slateblue": "#6A5ACD",
        "olivedrab": "#6B8E23",
        "slategray": "#708090",
        "slategrey": "#708090",
        "lightslategray": "#778899",
        "lightslategrey": "#778899",
        "mediumslateblue": "#7B68EE",
        "lawngreen": "#7CFC00",
        "chartreuse": "#7FFF00",
        "aquamarine": "#7FFFD4",
        "skyblue": "#87CEEB",
        "lightskyblue": "#87CEFA",
        "blueviolet": "#8A2BE2",
        "darkred": "#8B0000",
        "darkmagenta": "#8B008B",
        "saddlebrown": "#8B4513",
        "darkseagreen": "#8FBC8F",
        "lightgreen": "#90EE90",
        "mediumpurple": "#9370DB",
        "darkviolet": "#9400D3",
        "palegreen": "#98FB98",
        "darkorchid": "#9932CC",
        "yellowgreen": "#9ACD32",
        "sienna": "#A0522D",
        "brown": "#A52A2A",
        "darkgray": "#A9A9A9",
        "darkgrey": "#A9A9A9",
        "lightblue": "#ADD8E6",
        "greenyellow": "#ADFF2F",
        "paleturquoise": "#AFEEEE",
        "lightsteelblue": "#B0C4DE",
        "powderblue": "#B0E0E6",
        "firebrick": "#B22222",
        "darkgoldenrod": "#B8860B",
        "mediumorchid": "#BA55D3",
        "rosybrown": "#BC8F8F",
        "darkkhaki": "#BDB76B",
        "mediumvioletred": "#C71585",
        "indianred": "#CD5C5C",
        "peru": "#CD853F",
        "chocolate": "#D2691E",
        "tan": "#D2B48C",
        "lightgray": "#D3D3D3",
        "lightgrey": "#D3D3D3",
        "thistle": "#D8BFD8",
        "orchid": "#DA70D6",
        "goldenrod": "#DAA520",
        "palevioletred": "#DB7093",
        "crimson": "#DC143C",
        "gainsboro": "#DCDCDC",
        "plum": "#DDA0DD",
        "burlywood": "#DEB887",
        "lightcyan": "#E0FFFF",
        "lavender": "#E6E6FA",
        "darksalmon": "#E9967A",
        "violet": "#EE82EE",
        "palegoldenrod": "#EEE8AA",
        "lightcoral": "#F08080",
        "khaki": "#F0E68C",
        "aliceblue": "#F0F8FF",
        "honeydew": "#F0FFF0",
        "azure": "#F0FFFF",
        "sandybrown": "#F4A460",
        "wheat": "#F5DEB3",
        "beige": "#F5F5DC",
        "whitesmoke": "#F5F5F5",
        "mintcream": "#F5FFFA",
        "ghostwhite": "#F8F8FF",
        "salmon": "#FA8072",
        "antiquewhite": "#FAEBD7",
        "linen": "#FAF0E6",
        "lightgoldenrodyellow": "#FAFAD2",
        "oldlace": "#FDF5E6",
        "magenta": "#FF00FF",
        "deeppink": "#FF1493",
        "orangered": "#FF4500",
        "tomato": "#FF6347",
        "hotpink": "#FF69B4",
        "coral": "#FF7F50",
        "darkorange": "#FF8C00",
        "lightsalmon": "#FFA07A",
        "orange": "#FFA500",
        "lightpink": "#FFB6C1",
        "pink": "#FFC0CB",
        "gold": "#FFD700",
        "peachpuff": "#FFDAB9",
        "navajowhite": "#FFDEAD",
        "moccasin": "#FFE4B5",
        "bisque": "#FFE4C4",
        "mistyrose": "#FFE4E1",
        "blanchedalmond": "#FFEBCD",
        "papayawhip": "#FFEFD5",
        "lavenderblush": "#FFF0F5",
        "seashell": "#FFF5EE",
        "cornsilk": "#FFF8DC",
        "lemonchiffon": "#FFFACD",
        "floralwhite": "#FFFAF0",
        "snow": "#FFFAFA",
        "lightyellow": "#FFFFE0",
        "ivory": "#FFFFF0"
    }



#!/usr/bin/env python

""" MultiQC submodule to parse output from Picard VariantCallingMetrics """

import logging
from collections import OrderedDict
from multiqc.plots import bargraph

# Initialise the logger
log = logging.getLogger(__name__)


def parse_reports(parent_module):
    """ Find Picard VariantCallingMetrics reports and process their data """

    # get data
    data = collect_data(parent_module)

    # Filter to strip out ignored sample names
    data = parent_module.ignore_samples(data)

    # Reference data in parent module
    parent_module.picard_variantCalling_data = data

    if len(data) > 0:
        derive_data(data)

        # Write parsed data to a file
        parent_module.write_data_file(data, 'multiqc_picard_variantCalling')

        parent_module.general_stats_headers['DBSNP_TITV'] = {
            'title': 'TiTV ratio (known)',
            'description': "The Transition/Transversion ratio of the passing bi-allelic SNP calls made at SNP-database sites.",
            'min': 0,
            'scale': 'Blues',
            'shared_key': 'titv_ratio'
        }

        parent_module.general_stats_headers['NOVEL_TITV'] = {
            'title': 'TiTV ratio (novel)',
            'description': "The Transition/Transversion ratio of the passing bi-allelic SNP calls made at non-SNP-database sites.",
            'min': 0,
            'scale': 'Blues',
            'shared_key': 'titv_ratio'
        }

        parent_module.general_stats_headers['DBSNP_INS_DEL_RATIO'] = {
            'title': 'InDel ratio (known)',
            'description': "The Insertion / Deletion ratio of the passing bi-allelic SNP calls made at SNP-database sites.",
            'min': 0,
            'scale': 'Greens',
            'shared_key': 'indel_ratio',
            'hidden': True,
        }

        parent_module.general_stats_headers['NOVEL_INS_DEL_RATIO'] = {
            'title': 'InDel ratio (novel)',
            'description': "The Insertion / Deletion ratio of the passing bi-allelic SNP calls made at non-SNP-database sites.",
            'min': 0,
            'scale': 'Greens',
            'shared_key': 'indel_ratio',
            'hidden': True,
        }


        parent_module.general_stats_headers['total_called_variants_known'] = {
            'title': 'Called Variants (known)',
            'description': "Total counts of variants in SNP-database sites.",
            'shared_key': 'variant_count',
            'min': 0,
            'format': '{0:,.0f}',
            'hidden': True,
        }

        parent_module.general_stats_headers['total_called_variants_novel'] = {
            'title': 'Called Variants (novel)',
            'description': "Total counts of variants in non-SNP-database sites.",
            'shared_key': 'variant_count',
            'min': 0,
            'format': '{0:,.0f}',
            'hidden': True,
        }

        for s_name in data:
            if s_name not in parent_module.general_stats_data:
                parent_module.general_stats_data[s_name] = dict()
            parent_module.general_stats_data[s_name].update(data[s_name])


        # Variant Counts Bargraph
        parent_module.add_section(
            name='Variant Types',
            anchor='picard-variants-types',
            description='Variants that have been called, looking at variant types. Optionally filtered on label.',
            helptext="""
            Only passing variants are shown (i.e. non-filtered).\n
            SNPs are bi-allelic.\n
            Complex InDels are both an insertion and a deletion.
            """,
            plot=compare_variant_type_plot(data)
        )

        # Variant Counts Table
        parent_module.add_section(
            name='Variant Labels',
            anchor='picard-variants-labels',
            description='Variants that have been called, comparing with known variant sites.',
            helptext="""
            Only passing variants are shown (i.e. non-filtered).\n
            Variants contain bi-allelic SNPs, multi-allelic SNPs, simple and complex inserts and deletions.
            """,
            plot=compare_variants_label_plot(data)
        )

    return len(data)


def collect_data(parent_module):
    """ Find Picard VariantCallingMetrics reports and parse their data """

    data = dict()
    for file_meta in parent_module.find_log_files('picard/variant_calling_metrics', filehandles=True):
        s_name = None
        for header, value in table_in(file_meta['f'], pre_header_string='## METRICS CLASS'):
            if header == 'SAMPLE_ALIAS':
                s_name = value
                if s_name in data:
                    log.debug("Duplicate sample name found in {}! Overwriting: {}".format(file_meta['fn'], s_name))
                data[s_name] = OrderedDict()
            else:
                data[s_name][header] = value
    return data


def table_in(filehandle, pre_header_string):
    """ Generator that assumes a table starts the line after a given string """

    in_histogram = False
    next_is_header = False
    headers = list()
    for line in stripped(filehandle):
        if not in_histogram and line.startswith(pre_header_string):
            in_histogram = True
            next_is_header = True
        elif in_histogram and next_is_header:
            next_is_header = False
            headers = line.split("\t")
        elif in_histogram:
            values = line.split("\t")
            if values != ['']:
                for couple in zip(headers, values):
                    yield couple


def derive_data(data):
    """ Based on the data derive additional data """

    for s_name, values in data.items():
        # setup holding variable

        # Sum all variants that have been called
        total_called_variants = 0
        for value_name in ['TOTAL_SNPS', 'TOTAL_COMPLEX_INDELS', 'TOTAL_MULTIALLELIC_SNPS', 'TOTAL_INDELS']:
            total_called_variants = total_called_variants + int(values[value_name])
        values['total_called_variants'] = total_called_variants

        # Sum all variants that have been called and are known
        total_called_variants_known = 0
        for value_name in ['NUM_IN_DB_SNP', 'NUM_IN_DB_SNP_COMPLEX_INDELS', 'NUM_IN_DB_SNP_MULTIALLELIC']:
            total_called_variants_known = total_called_variants_known + int(values[value_name])
        total_called_variants_known = total_called_variants_known + int(values['TOTAL_INDELS']) - int(values['NOVEL_INDELS'])
        values['total_called_variants_known'] = total_called_variants_known

        # Extrapolate the total novel variants
        values['total_called_variants_novel'] = total_called_variants - total_called_variants_known


def stripped(iterator):
    """ Generator to strip string of whitespace """
    for item in iterator:
        yield item.strip()


def compare_variant_type_plot(data):
    """ Return HTML for the Variant Counts barplot """
    keys = OrderedDict()
    keys['snps'] = {'name': 'SNPs', 'color': '#7cb5ec'}
    keys['indels'] = {'name': 'InDels', 'color': '#90ed7d'}
    keys['multiallelic_snps'] = {'name': 'multi-allelic SNP', 'color': 'orange'}
    keys['complex_indels'] = {'name': 'Complex InDels', 'color': '#8085e9'}

    total_variants = dict()
    known_variants = dict()
    novel_variants = dict()
    for s_name, values in data.items():
        total_variants[s_name] = {
            'snps': values['TOTAL_SNPS'],
            'indels': values['TOTAL_INDELS'],
            'multiallelic_snps': values['TOTAL_MULTIALLELIC_SNPS'],
            'complex_indels': values['TOTAL_COMPLEX_INDELS'],
        }

        known_variants[s_name] = {
            'snps': values['NUM_IN_DB_SNP'],
            'indels': int(values['TOTAL_INDELS'])-int(values['NOVEL_INDELS']),
            'multiallelic_snps': values['NUM_IN_DB_SNP_MULTIALLELIC'],
            'complex_indels': values['NUM_IN_DB_SNP_COMPLEX_INDELS'],
        }

        novel_variants[s_name] = {
            'snps': values['NOVEL_SNPS'],
            'indels': values['NOVEL_INDELS'],
            'multiallelic_snps': int(values['TOTAL_MULTIALLELIC_SNPS'])-int(values['NUM_IN_DB_SNP_MULTIALLELIC']),
            'complex_indels': int(values['TOTAL_COMPLEX_INDELS'])-int(values['NUM_IN_DB_SNP_COMPLEX_INDELS']),
        }

    plot_conf = {
        'id': 'picard_variantCallingMetrics_variant_type',
        'title': 'Picard: Variants Called',
        'ylab': 'Counts of Variants',
        'hide_zero_cats': False,
        'data_labels': [
            {'name': 'Total'},
            {'name': 'Known'},
            {'name': 'Novel'}
        ],
    }
    return bargraph.plot(data=[total_variants, known_variants, novel_variants],
                         cats=[keys, keys, keys],
                         pconfig=plot_conf)


def compare_variants_label_plot(data):
    """ Return HTML for the Compare variants plot"""
    keys = OrderedDict()

    keys['total_called_variants_known'] = {'name': 'Known Variants'}
    keys['total_called_variants_novel'] = {'name': 'Novel Variants'}

    pconfig = {
        'id': 'picard_variantCallingMetrics_variant_label',
        'title': 'Picard: Variants Called',
        'ylab': 'Counts of Variants',
     }

    return bargraph.plot(data, cats=keys, pconfig=pconfig)


def merge_two_dicts(x, y):
    z = x.copy()   # start with x's keys and values
    z.update(y)    # modifies z with y's keys and values & returns None
    return z



#!/usr/bin/env python

""" MultiQC submodule to parse output from deepTools bamPEFragmentSize for summary table """

import logging
from collections import OrderedDict

from multiqc.plots import table, linegraph

# Initialise the logger
log = logging.getLogger(__name__)

class bamPEFragmentSizeTableMixin():
    def parse_bamPEFragmentSize(self):
        """Find bamPEFragmentSize output. Supports the --table option"""
        self.deeptools_bamPEFragmentSize = dict()
        for f in self.find_log_files('deeptools/bamPEFragmentSizeTable'):
            parsed_data = self.parseBamPEFile(f)
            for k, v in parsed_data.items():
                if k in self.deeptools_bamPEFragmentSize:
                    log.warning("Replacing duplicate sample {}.".format(k))
                self.deeptools_bamPEFragmentSize[k] = v

            if len(parsed_data) > 0:
                self.add_data_source(f, section='bamPEFragmentSize')

        self.deeptools_bamPEFragmentSize = self.ignore_samples(self.deeptools_bamPEFragmentSize)

        if len(self.deeptools_bamPEFragmentSize) > 0:
            headersSE = OrderedDict()
            headersSE["Reads Sampled"] = {
                'title': '# Sampled',
                'description': 'Number of reads sampled',
                'format': '{:,.0f}'
            }
            headersSE["Read Len. Min."] = {
                'title': 'Min',
                'description': 'Minimum read length',
                'format': '{:,.0f}',
                'shared_key': 'read_length'
            }
            headersSE["Read Len. 1st. Qu."] = {
                'title': '1st Quartile',
                'description': '1st quartile read length',
                'format': '{:,.0f}',
                'shared_key': 'read_length'
            }
            headersSE["Read Len. Mean"] = {
                'title': 'Mean',
                'description': 'Mean read length',
                'shared_key': 'read_length'
            }
            headersSE["Read Len. Median"] = {
                'title': 'Median',
                'description': 'Median read length',
                'format': '{:,.0f}',
                'shared_key': 'read_length'
            }
            headersSE["Read Len. 3rd Qu."] = {
                'title': '3rd Quartile',
                'description': '3rd quartile read length',
                'format': '{:,.0f}',
                'shared_key': 'read_length'
            }
            headersSE["Read Len. Max"] = {
                'title': 'Max',
                'description': 'Maximum read length',
                'format': '{:,.0f}',
                'shared_key': 'read_length'
            }
            headersSE["Read Len. Std."] = {
                'title': 'Std. Dev.',
                'description': 'read length standard deviation',
                'shared_key': 'read_length'
            }
            headersSE["Read Med. Abs. Dev."] = {
                'title': 'MAD',
                'description': 'read length median absolute deviation',
                'shared_key': 'read_length'
            }
            config = {'namespace': 'deepTools bamPEFragmentSize'}
            self.add_section(
                name = "Read length metrics",
                anchor = "deeptools_readlengths",
                plot = table.plot(self.deeptools_bamPEFragmentSize, headersSE, config)
            )

            headersPE = OrderedDict()
            headersPE["Frag. Sampled"] = {
                'title': '# Sampled',
                'description': 'Number of fragments sampled',
                'format': '{:,.0f}'
            }
            headersPE["Frag. Len. Min."] = {
                'title': 'Min',
                'description': 'Minimum fragment length',
                'format': '{:,.0f}',
                'shared_key': 'frag_length'
            }
            headersPE["Frag. Len. 1st. Qu."] = {
                'title': '1st Quartile',
                'description': '1st quartile fragment length',
                'format': '{:,.0f}',
                'shared_key': 'frag_length'
            }
            headersPE["Frag. Len. Mean"] = {
                'title': 'Mean',
                'description': 'Mean fragment length',
                'format': '{:,.0f}',
                'shared_key': 'frag_length'
            }
            headersPE["Frag. Len. Median"] = {
                'title': 'Median',
                'description': 'Median fragment length',
                'format': '{:,.0f}',
                'shared_key': 'frag_length'
            }
            headersPE["Frag. Len. 3rd Qu."] = {
                'title': '3rd Quartile',
                'description': '3rd quartile fragment length',
                'format': '{:,.0f}',
                'shared_key': 'frag_length'
            }
            headersPE["Frag. Len. Max"] = {
                'title': 'Max',
                'description': 'Maximum fragment length',
                'format': '{:,.0f}',
                'shared_key': 'frag_length'
            }
            headersPE["Frag. Len. Std."] = {
                'title': 'Std. Dev.',
                'description': 'Fragment length standard deviation',
                'shared_key': 'frag_length'
            }
            headersPE["Frag. Med. Abs. Dev."] = {
                'title': 'MAD',
                'description': 'Fragment length median absolute deviation',
                'shared_key': 'frag_length'
            }

            # Are there any PE datasets?
            PE = False
            for k, v in self.deeptools_bamPEFragmentSize.items():
                if 'Frag. Len. Min.' in v:
                    PE = True
                    break
            if PE:
                self.add_section(name="Fragment length metrics",
                                 anchor="deeptools_fragmentlengths",
                                 plot=table.plot(self.deeptools_bamPEFragmentSize, headersPE, config))

            # Read length plot
            config = {
                'data_labels': [
                    {
                        'name': "Read length distribution",
                        'title': "Read length distribution",
                        'ylab': "Read length (bases)"
                    },
                    {
                        'name': "Fragment length distribution",
                        'title': "Fragment length distribution",
                        'ylab': "Fragment length (bases)"
                    }
                ],
                'id': 'deeptools_readlengthsPlot',
                'title': 'deepTools: Read/Fragment length distribution',
                'namespace': 'deepTools bamPEFragmentSize',
                'ylab': "Read length (bases)",
                'xlab': "Percentile"
            }
            SE = dict()
            PE = dict()
            for k, v in self.deeptools_bamPEFragmentSize.items():
                SE[k] = {0: v['Read Len. Min.'],
                         10: v['Read Len. 10%'],
                         20: v['Read Len. 20%'],
                         25: v['Read Len. 1st. Qu.'],
                         30: v['Read Len. 30%'],
                         40: v['Read Len. 40%'],
                         50: v['Read Len. Median'],
                         60: v['Read Len. 60%'],
                         70: v['Read Len. 70%'],
                         75: v['Read Len. 3rd Qu.'],
                         80: v['Read Len. 80%'],
                         90: v['Read Len. 90%'],
                         99: v['Read Len. 99%'],
                         100: v['Read Len. Max']}
                if 'Frag. Len. Min.' not in v:
                    continue
                PE[k] = {0: v['Frag. Len. Min.'],
                         10: v['Frag. Len. 10%'],
                         20: v['Frag. Len. 20%'],
                         25: v['Frag. Len. 1st. Qu.'],
                         30: v['Frag. Len. 30%'],
                         40: v['Frag. Len. 40%'],
                         50: v['Frag. Len. Median'],
                         60: v['Frag. Len. 60%'],
                         70: v['Frag. Len. 70%'],
                         75: v['Frag. Len. 3rd Qu.'],
                         80: v['Frag. Len. 80%'],
                         90: v['Frag. Len. 90%'],
                         99: v['Frag. Len. 99%'],
                         100: v['Frag. Len. Max']}
            self.add_section(
                name = "Read/fragment length distribution",
                anchor = "deeptools_fragmentlengths_dist",
                plot = linegraph.plot([SE, PE], config)
            )

        return len(self.deeptools_bamPEFragmentSize)

    def parseBamPEFile(self, f):
        d = {}
        headers = None
        for line in f['f'].splitlines():
            cols = line.rstrip().split("\t")
            if headers is None:
                headers = cols
            else:
                s_name = None
                for idx, h in enumerate(headers):
                    if idx == 0:
                        s_name = self.clean_s_name(cols[0], f['root'])
                        if s_name in d:
                            log.debug("Replacing duplicate sample {}.".format(s_name))
                        d[s_name] = OrderedDict()
                    else:
                        if idx < 19 and cols[1] == "0":
                            # Don't store fragment metrics for SE datasets, they're just 0.
                            continue
                        try:
                            # Most values are ac
                            d[s_name][h] = self._int(cols[idx])
                        except ValueError:
                            d[s_name][h] = float(cols[idx])

        return d



#!/usr/bin/env python

""" MultiQC module to parse output from Fastp """

from __future__ import print_function
from collections import OrderedDict
import logging
import json

from multiqc import config
from multiqc.plots import bargraph, linegraph
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    fastp module class
    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='fastp', anchor='fastp',
            href='https://github.com/OpenGene/fastp',
            info="An ultra-fast all-in-one FASTQ preprocessor (QC, adapters, trimming, filtering, splitting...)")

        # Find and load any fastp reports
        self.fastp_data = dict()
        self.fastp_duplication_plotdata = dict()
        self.fastp_insert_size_data = dict()
        self.fastp_all_data = dict()
        self.fastp_qual_plotdata = dict()
        self.fastp_gc_content_data = dict()
        self.fastp_n_content_data = dict()
        for k in ['read1_before_filtering','read2_before_filtering','read1_after_filtering','read2_after_filtering']:
            self.fastp_qual_plotdata[k] = dict()
            self.fastp_gc_content_data[k] = dict()
            self.fastp_n_content_data[k] = dict()

        for f in self.find_log_files('fastp', filehandles=True):
            self.parse_fastp_log(f)

        # Filter to strip out ignored sample names
        self.fastp_data = self.ignore_samples(self.fastp_data)

        if len(self.fastp_data) == 0:
            raise UserWarning

        log.info("Found {} reports".format(len(self.fastp_data)))

        # Write parsed report data to a file
        ## Parse whole JSON to save all its content
        self.write_data_file(self.fastp_all_data, 'multiqc_fastp')

        # General Stats Table
        self.fastp_general_stats_table()

        # Filtering statistics bar plot
        self.add_section (
           name = 'Filtered Reads',
           anchor = 'fastp-filtered-reads-chart',
           description = 'Filtering statistics of sampled reads.',
           plot = self.fastp_filtered_reads_chart()
        )

        # Duplication rate plot
        if len(self.fastp_duplication_plotdata) > 0:
            self.add_section (
                name = 'Duplication Rates',
                anchor = 'fastp-duprates',
                description = 'Duplication rates of sampled reads.',
                plot = linegraph.plot(
                    self.fastp_duplication_plotdata,
                    {
                        'id': 'fastp-duprates-plot',
                        'title': 'Fastp: Duplication Rate',
                        'xlab': 'Duplication level',
                        'ylab': 'Read percent',
                        'yCeiling': 100,
                        'ymin': 0,
                        'yLabelFormat': '{value}%',
                        'tt_label': '{point.x}: {point.y:.2f}%',
                        'xDecimals': False
                    }
                )
            )

        # Insert size plot
        if len(self.fastp_insert_size_data) > 0:
            self.add_section (
                name = 'Insert Sizes',
                anchor = 'fastp-insert-size',
                description = 'Insert size estimation of sampled reads.',
                plot = linegraph.plot(
                    self.fastp_insert_size_data,
                    {
                        'id': 'fastp-insert-size-plot',
                        'title': 'Fastp: Insert Size Distribution',
                        'xlab': 'Insert size',
                        'ylab': 'Read percent',
                        'yCeiling': 100,
                        'ymin': 0,
                        'yLabelFormat': '{value}%',
                        'tt_label': '{point.x}: {point.y:.2f}%',
                        'xDecimals': False,
                        'smooth_points': 300,
                        'smooth_points_sumcounts': False
                    }
                )
            )

        # Base quality plot
        self.add_section (
            name = 'Sequence Quality',
            anchor = 'fastp-seq-quality',
            description = 'Average sequencing quality over each base of all reads.',
            plot = self.fastp_read_qual_plot()
        )

        # GC content plot
        self.add_section (
            name = 'GC Content',
            anchor = 'fastp-seq-content-gc',
            description = 'Average GC content over each base of all reads.',
            plot = self.fastp_read_gc_plot()
        )

        # N content plot
        self.add_section (
            name = 'N content',
            anchor = 'fastp-seq-content-n',
            description = 'Average N content over each base of all reads.',
            plot = self.fastp_read_n_plot()
        )


    def parse_fastp_log(self, f):
        """ Parse the JSON output from fastp and save the summary statistics """
        try:
            parsed_json = json.load(f['f'])
        except:
            log.warning("Could not parse fastp JSON: '{}'".format(f['fn']))
            return None

        # Fetch a sample name from the command
        s_name = f['s_name']
        cmd = parsed_json['command'].split()
        for i,v in enumerate(cmd):
            if v == '-i':
                s_name = self.clean_s_name(cmd[i+1], f['root'])
        if s_name == 'fastp':
            log.warning('Could not parse sample name from fastp command: {}'.format(f['fn']))

        self.add_data_source(f, s_name)
        self.fastp_data[s_name] = {}
        self.fastp_duplication_plotdata[s_name] = {}
        self.fastp_insert_size_data[s_name] = {}
        self.fastp_all_data[s_name] = parsed_json
        for k in ['read1_before_filtering','read2_before_filtering','read1_after_filtering','read2_after_filtering']:
            self.fastp_qual_plotdata[k][s_name] = {}
            self.fastp_gc_content_data[k][s_name] = {}
            self.fastp_n_content_data[k][s_name] = {}

        # Parse filtering_result
        try:
            for k in parsed_json['filtering_result']:
                self.fastp_data[s_name]['filtering_result_{}'.format(k)] = float(parsed_json['filtering_result'][k])
        except KeyError:
            log.debug("fastp JSON did not have 'filtering_result' key: '{}'".format(f['fn']))

        # Parse duplication
        try:
            self.fastp_data[s_name]['pct_duplication'] = float(parsed_json['duplication']['rate'] * 100.0)
        except KeyError:
            log.debug("fastp JSON did not have a 'duplication' key: '{}'".format(f['fn']))

        # Parse after_filtering
        try:
            for k in parsed_json['summary']['after_filtering']:
                self.fastp_data[s_name]['after_filtering_{}'.format(k)] = float(parsed_json['summary']['after_filtering'][k])
        except KeyError:
            log.debug("fastp JSON did not have a 'summary'-'after_filtering' keys: '{}'".format(f['fn']))


        # Parse data required to calculate Pct reads surviving
        try:
            self.fastp_data[s_name]['before_filtering_total_reads'] = float(parsed_json['summary']['before_filtering']['total_reads'])
        except KeyError:
            log.debug("Could not find pre-filtering # reads: '{}'".format(f['fn']))

        try:
            self.fastp_data[s_name]['pct_surviving'] = (self.fastp_data[s_name]['after_filtering_total_reads'] / self.fastp_data[s_name]['before_filtering_total_reads']) * 100.0
        except KeyError:
            log.debug("Could not calculate 'pct_surviving': {}".format(f['fn']))

        # Parse adapter_cutting
        try:
            for k in parsed_json['adapter_cutting']:
                try:
                    self.fastp_data[s_name]['adapter_cutting_{}'.format(k)] = float(parsed_json['adapter_cutting'][k])
                except (ValueError, TypeError):
                    pass
        except KeyError:
            log.debug("fastp JSON did not have a 'adapter_cutting' key, skipping: '{}'".format(f['fn']))

        try:
            self.fastp_data[s_name]['pct_adapter'] = (self.fastp_data[s_name]['adapter_cutting_adapter_trimmed_reads'] / self.fastp_data[s_name]['before_filtering_total_reads']) * 100.0
        except KeyError:
            log.debug("Could not calculate 'pct_adapter': {}".format(f['fn']))

        # Duplication rate plot data
        try:
            # First count the total read count in the dup analysis
            total_reads = 0
            for v in parsed_json['duplication']['histogram']:
                total_reads += v
            if total_reads == 0:
                raise KeyError
            # Calculate percentages
            for i, v in enumerate(parsed_json['duplication']['histogram']):
                self.fastp_duplication_plotdata[s_name][i+1] = (float(v) / float(total_reads)) * 100.0
        except KeyError:
            log.debug("No duplication rate plot data: {}".format(f['fn']))

        # Insert size plot data
        try:
            # First count the total read count in the insert size analysis
            total_reads = 0
            max_i = 0
            for i, v in enumerate(parsed_json['insert_size']['histogram']):
                total_reads += v
                if float(v) > 0:
                    max_i = i
            if total_reads == 0:
                raise KeyError
            # Calculate percentages
            for i, v in enumerate(parsed_json['insert_size']['histogram']):
                if i <= max_i:
                    self.fastp_insert_size_data[s_name][i+1] = (float(v) / float(total_reads)) * 100.0
        except KeyError:
            log.debug("No insert size plot data: {}".format(f['fn']))

        for k in ['read1_before_filtering','read2_before_filtering','read1_after_filtering','read2_after_filtering']:
            # Read quality data
            try:
                for i, v in enumerate(parsed_json[k]['quality_curves']['mean']):
                    self.fastp_qual_plotdata[k][s_name][i+1] = float(v)
            except KeyError:
                log.debug("Read quality {} not found: {}".format(k, f['fn']))

            # GC and N content plots
            try:
                for i, v in enumerate(parsed_json[k]['content_curves']['GC']):
                    self.fastp_gc_content_data[k][s_name][i+1] = float(v) * 100.0
                for i, v in enumerate(parsed_json[k]['content_curves']['N']):
                    self.fastp_n_content_data[k][s_name][i+1] = float(v) * 100.0
            except KeyError:
                log.debug("Content curve data {} not found: {}".format(k, f['fn']))


        # Remove empty dicts
        if len(self.fastp_data[s_name]) == 0:
            del self.fastp_data[s_name]
        if len(self.fastp_duplication_plotdata[s_name]) == 0:
            del self.fastp_duplication_plotdata[s_name]
        if len(self.fastp_insert_size_data[s_name]) == 0:
            del self.fastp_insert_size_data[s_name]
        if len(self.fastp_all_data[s_name]) == 0:
            del self.fastp_all_data[s_name]
        # Don't delete dicts with subkeys, messes up multi-panel plots


    def fastp_general_stats_table(self):
        """ Take the parsed stats from the fastp report and add it to the
        General Statistics table at the top of the report """

        headers = OrderedDict()
        headers['pct_duplication'] = {
            'title': '% Duplication',
            'description': 'Duplication rate in filtered reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'RdYlGn-rev'
        }
        headers['after_filtering_q30_rate'] = {
            'title': '% > Q30',
            'description': 'Percentage of reads > Q30 after filtering',
            'min': 0,
            'max': 100,
            'modify': lambda x: x * 100.0,
            'scale': 'GnBu',
            'suffix': '%',
            'hidden': True
        }
        headers['after_filtering_q30_bases'] = {
            'title': '{} Q30 bases'.format(config.base_count_prefix),
            'description': 'Bases > Q30 after filtering ({})'.format(config.base_count_desc),
            'min': 0,
            'modify': lambda x: x * config.base_count_multiplier,
            'scale': 'GnBu',
            'shared_key': 'base_count',
            'hidden': True
        }
        headers['after_filtering_gc_content'] = {
            'title': 'GC content',
            'description': 'GC content after filtering',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'Blues',
            'modify': lambda x: x * 100.0
        }
        headers['pct_surviving'] = {
            'title': '% PF',
            'description': 'Percent reads passing filter',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'BuGn',
        }
        headers['pct_adapter'] = {
            'title': '% Adapter',
            'description': 'Percentage adapter-trimmed reads',
            'max': 100,
            'min': 0,
            'suffix': '%',
            'scale': 'RdYlGn-rev',
        }

        self.general_stats_addcols(self.fastp_data, headers)

    def fastp_filtered_reads_chart(self):
        """ Function to generate the fastp filtered reads bar plot """
        # Specify the order of the different possible categories
        keys = OrderedDict()
        keys['filtering_result_passed_filter_reads'] =  { 'name': 'Passed Filter' }
        keys['filtering_result_low_quality_reads'] =    { 'name': 'Low Quality' }
        keys['filtering_result_too_many_N_reads'] =     { 'name': 'Too Many N' }
        keys['filtering_result_too_short_reads'] =      { 'name': 'Too short' }

        # Config for the plot
        pconfig = {
            'id': 'fastp_filtered_reads_plot',
            'title': 'Fastp: Filtered Reads',
            'ylab': '# Reads',
            'cpswitch_counts_label': 'Number of Reads',
            'hide_zero_cats': False,
        }
        return bargraph.plot(self.fastp_data, keys, pconfig)

    def fastp_read_qual_plot(self):
        """ Make the read quality plot for Fastp """
        data_labels, pdata = self.filter_pconfig_pdata_subplots(self.fastp_qual_plotdata, 'Sequence Quality')
        pconfig = {
            'id': 'fastp-seq-quality-plot',
            'title': 'Fastp: Sequence Quality',
            'xlab': 'Read Position',
            'ylab': 'R1 Before filtering: Sequence Quality',
            'ymin': 0,
            'xDecimals': False,
            'data_labels': data_labels
        }
        return linegraph.plot(pdata, pconfig)

    def fastp_read_gc_plot(self):
        """ Make the read GC plot for Fastp """
        data_labels, pdata = self.filter_pconfig_pdata_subplots(self.fastp_gc_content_data, 'Base Content Percent')
        pconfig = {
            'id': 'fastp-seq-content-gc-plot',
            'title': 'Fastp: Read GC Content',
            'xlab': 'Read Position',
            'ylab': 'R1 Before filtering: Base Content Percent',
            'ymax': 100,
            'ymin': 0,
            'xDecimals': False,
            'yLabelFormat': '{value}%',
            'tt_label': '{point.x}: {point.y:.2f}%',
            'data_labels': data_labels
        }
        return linegraph.plot(pdata, pconfig)

    def fastp_read_n_plot(self):
        """ Make the read N content plot for Fastp """
        data_labels, pdata = self.filter_pconfig_pdata_subplots(self.fastp_n_content_data, 'Base Content Percent')
        pconfig = {
            'id': 'fastp-seq-content-n-plot',
            'title': 'Fastp: Read N Content',
            'xlab': 'Read Position',
            'ylab': 'R1 Before filtering: Base Content Percent',
            'yCeiling': 100,
            'yMinRange': 5,
            'ymin': 0,
            'xDecimals': False,
            'yLabelFormat': '{value}%',
            'tt_label': '{point.x}: {point.y:.2f}%',
            'data_labels': data_labels
        }
        return linegraph.plot(pdata, pconfig)

    def filter_pconfig_pdata_subplots(self, data, label):
        data_labels = []
        pdata = []
        config = {
            'read1_before_filtering': {'name': 'Read 1: Before filtering', 'ylab': 'R1 Before filtering: {}'.format(label)},
            'read1_after_filtering':  {'name': 'Read 1: After filtering',  'ylab': 'R1 After filtering: {}'.format(label)},
            'read2_before_filtering': {'name': 'Read 2: Before filtering', 'ylab': 'R2 Before filtering: {}'.format(label)},
            'read2_after_filtering':  {'name': 'Read 2: After filtering',  'ylab': 'R2 After filtering: {}'.format(label)},
        }
        for k in config:
            if sum([len(data[k][x]) for x in data[k]]) > 0:
                data_labels.append(config[k])
                pdata.append(data[k])

        return data_labels, pdata



#!/usr/bin/env python

""" MultiQC functions to plot a heatmap """

from __future__ import print_function
import logging
import random

from multiqc.utils import config, report

logger = logging.getLogger(__name__)

letters = 'abcdefghijklmnopqrstuvwxyz'

def plot (data, xcats, ycats=None, pconfig=None):
    """ Plot a 2D heatmap.
    :param data: List of lists, each a representing a row of values.
    :param xcats: Labels for x axis
    :param ycats: Labels for y axis. Defaults to same as x.
    :param pconfig: optional dict with config key:value pairs.
    :return: HTML and JS, ready to be inserted into the page
    """

    if pconfig is None:
        pconfig = {}

    # Allow user to overwrite any given config for this plot
    if 'id' in pconfig and pconfig['id'] and pconfig['id'] in config.custom_plot_config:
        for k, v in config.custom_plot_config[pconfig['id']].items():
            pconfig[k] = v

    if ycats is None:
        ycats = xcats

    # Make a plot
    return highcharts_heatmap(data, xcats, ycats, pconfig)



def highcharts_heatmap (data, xcats, ycats, pconfig=None):
    """
    Build the HTML needed for a HighCharts line graph. Should be
    called by plot_xy_data, which properly formats input data.
    """
    if pconfig is None:
        pconfig = {}

    # Reformat the data for highcharts
    pdata = []
    for i, arr in enumerate(data):
        for j, val in enumerate(arr):
            pdata.append([j,i,val])

    # Get the plot ID
    if pconfig.get('id') is None:
        pconfig['id'] = 'mqc_hcplot_'+''.join(random.sample(letters, 10))

    # Sanitise plot ID and check for duplicates
    pconfig['id'] = report.save_htmlid(pconfig['id'])

    # Build the HTML for the page
    html = '<div class="mqc_hcplot_plotgroup">'

    # The 'sort by highlights button'
    html += """<div class="btn-group hc_switch_group">
        <button type="button" class="mqc_heatmap_sortHighlight btn btn-default btn-sm" data-target="#{id}" disabled="disabled">
            <span class="glyphicon glyphicon-sort-by-attributes-alt"></span> Sort by highlight
        </button>
    </div>""".format(id=pconfig['id'])

    # The plot div
    html += '<div class="hc-plot-wrapper"><div id="{id}" class="hc-plot not_rendered hc-heatmap"><small>loading..</small></div></div></div> \n'.format(id=pconfig['id'])

    report.num_hc_plots += 1

    report.plot_data[pconfig['id']] = {
        'plot_type': 'heatmap',
        'data': pdata,
        'xcats': xcats,
        'ycats': ycats,
        'config': pconfig
    }

    return html



""" MultiQC module to parse output from mosdepth """

from __future__ import print_function

from collections import defaultdict, OrderedDict
import logging

from multiqc import config
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
from multiqc.modules.qualimap.QM_BamQC import coverage_histogram_helptext, genome_fraction_helptext
from multiqc.plots import linegraph
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
    Mosdepth can generate multiple outputs with a common prefix and different endings.
    The module can use first 2 (preferring "region" if exists, otherwise "global"),
    to build 2 plots: coverage distribution and per-contig average coverage.

    {prefix}.mosdepth.global.dist.txt
    a distribution of proportion of bases covered at or above a given threshhold for each chromosome and genome-wide

    1       2       0.00
    1       1       0.00
    1       0       1.00
    total   2       0.00
    total   1       0.00
    total   0       1.00

    {prefix}.mosdepth.region.dist.txt (if --by is specified)
    same, but in regions

    1       2       0.01
    1       1       0.01
    1       0       1.00
    total   2       0.00
    total   1       0.00
    total   0       1.00

    {prefix}.per-base.bed.gz (unless -n/--no-per-base is specified)

    1       0       881481  0
    1       881481  881482  2
    1       881482  881485  4

    {prefix}.regions.bed.gz (if --by is specified)
    the mean per-region from either a BED file or windows of specified size

    1       2488047 2488227 TNFRSF14        0.00
    1       2489098 2489338 TNFRSF14        0.00

    {prefix}.quantized.bed.gz (if --quantize is specified)
    quantized output that merges adjacent bases as long as they fall in the same coverage bins e.g. (10-20)

    1       0       881481  0:1
    1       881481  881485  1:5
    1       881485  881769  5:150

    {prefix}.thresholds.bed.gz (if --thresholds is specified) - how many bases in each region are covered at the given thresholds

    #chrom  start   end     region     1X      10X     20X     30X
    1       2488047 2488227 TNFRSF14   0       0       0       0
    1       2489098 2489338 TNFRSF14   0       0       0       0

    """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='mosdepth', anchor='mosdepth',
            href="https://github.com/brentp/mosdepth",
            info="performs fast BAM/CRAM depth calculation for WGS, exome, or targeted sequencing")

        dist_data, cov_data, xmax, perchrom_avg_data = self.parse_cov_dist()

        # Filter out any samples from --ignore-samples
        dist_data = self.ignore_samples(dist_data)
        cov_data = self.ignore_samples(cov_data)
        perchrom_avg_data = self.ignore_samples(perchrom_avg_data)

        # No samples found
        num_samples = max(
            len(dist_data),
            len(cov_data),
            len(perchrom_avg_data)
        )
        if num_samples == 0:
            raise UserWarning
        log.info("Found {} reports".format(num_samples))

        if dist_data:
            self.add_section(
                name='Coverage distribution',
                anchor='mosdepth-coverage-dist',
                description='Distribution of the number of locations in the reference genome with a given depth of coverage',
                helptext=genome_fraction_helptext,
                plot=linegraph.plot(dist_data, {
                    'id': 'mosdepth-coverage-dist-id',
                    'title': 'Mosdepth: Coverage Distribution',
                    'xlab': 'Coverage (X)',
                    'ylab': '% bases in genome/regions covered by at least X reads',
                    'ymax': 100,
                    'xmax': xmax,
                    'tt_label': '<b>{point.x}X</b>: {point.y:.2f}%',
                    'smooth_points': 500,
                })
            )
        if cov_data:
            self.add_section(
                name='Coverage plot',
                anchor='mosdepth-coverage-cov',
                description='Number of locations in the reference genome with a given depth of coverage',
                helptext=coverage_histogram_helptext,
                plot=linegraph.plot(cov_data, {
                    'id': 'mosdepth-coverage-plot-id',
                    'title': 'Mosdepth: Coverage Depth',
                    'xlab': 'Coverage (X)',
                    'ylab': '% bases in genome/regions covered at X reads',
                    'ymax': 100,
                    'xmax': xmax,
                    'tt_label': '<b>{point.x}X</b>: {point.y:.2f}%',
                    'smooth_points': 500,
                })
            )
        if perchrom_avg_data:
            self.add_section(
                name='Average coverage per contig',
                anchor='mosdepth-coverage-per-contig-id',
                description='Average coverage per contig or chromosome',
                plot=linegraph.plot(perchrom_avg_data, {
                    'id': 'mosdepth-coverage-per-contig',
                    'title': 'Mosdepth: Coverage per contig',
                    'xlab': 'region',
                    'ylab': 'average coverage',
                    'categories': True,
                    'tt_label': '<b>{point.x}</b>: {point.y:.1f}x',
                    'smooth_points': 500,
                })
            )
        if dist_data:
            threshs, hidden_threshs = get_cov_thresholds()
            self.genstats_cov_thresholds(dist_data, threshs, hidden_threshs)
            self.genstats_mediancov(dist_data)

    def parse_cov_dist(self):
        dist_data = defaultdict(OrderedDict)  # cumulative distribution
        cov_data = defaultdict(OrderedDict)  # absoulte (non-cumulative) coverage
        xmax = 0
        perchrom_avg_data = defaultdict(OrderedDict)  # per chromosome average coverage

        for scope in ('region', 'global'):
            for f in self.find_log_files('mosdepth/' + scope + '_dist'):
                s_name = self.clean_s_name(f['fn'], root=None).replace('.mosdepth.' + scope + '.dist', '')
                if s_name in dist_data:  # both region and global might exist, prioritizing region
                    continue

                for line in f['f'].split("\n"):
                    if "\t" not in line:
                        continue
                    contig, cutoff_reads, bases_fraction = line.split("\t")
                    if contig == "total":  # for global coverage distribution
                        cumcov = 100.0 * float(bases_fraction)
                        x = int(cutoff_reads)
                        dist_data[s_name][x] = cumcov
                        # converting cumulative coverage into absoulte coverage:
                        """
                        *example*              x:  cumcov:  abscov:
                        3x                     3x  0      =               0
                        2x     -               2x  0.10   = 0.10 - 0    = 0.10
                        1x     --------        1x  0.80   = 0.80 - 0.10 = 0.70
                        genome ..........      0x  1.00   = 1.00 - 0.80 = 0.20
                        """
                        if x + 1 not in dist_data[s_name]:
                            cov_data[s_name][x] = cumcov
                        else:
                            cov_data[s_name][x] = cumcov - dist_data[s_name][x + 1]
                        if cumcov > 1:  # require >1% to prevent long flat tail
                            xmax = max(xmax, x)
                    else:  # for per-contig plot
                        avg = perchrom_avg_data[s_name].get(contig, 0) + float(bases_fraction)
                        perchrom_avg_data[s_name][contig] = avg

                if s_name in dist_data:
                    self.add_data_source(f, s_name=s_name, section='genome_results')

        return dist_data, cov_data, xmax, perchrom_avg_data

    def genstats_cov_thresholds(self, dist_data, threshs, hidden_threshs):
        data = defaultdict(OrderedDict)
        for s_name, d in dist_data.items():
            dist_subset = {t: data for t, data in d.items() if t in threshs}
            for t in threshs:
                if int(t) in dist_subset:
                    data[s_name]['{}_x_pc'.format(t)] = dist_subset[t]
                else:
                    data[s_name]['{}_x_pc'.format(t)] = 0

        headers = OrderedDict()
        for t in threshs:
            headers['{}_x_pc'.format(t)] = {
                'title': '&ge; {}X'.format(t),
                'description': 'Fraction of genome with at least {}X coverage'.format(t),
                'max': 100,
                'min': 0,
                'suffix': '%',
                'scale': 'RdYlGn',
                'hidden': t in hidden_threshs,
            }
        self.general_stats_addcols(data, headers)

    def genstats_mediancov(self, dist_data):
        data = defaultdict(OrderedDict)
        for s_name, d in dist_data.items():
            median_cov = None
            for this_cov, cum_pct in d.items():
                if cum_pct >= 50:
                    median_cov = this_cov
                    break
            data[s_name]['median_coverage'] = median_cov

        headers = OrderedDict()
        headers['median_coverage'] = {
            'title': 'Median',
            'description': 'Median coverage',
            'min': 0,
            'suffix': 'X',
            'scale': 'BuPu'
        }
        self.general_stats_addcols(data, headers)


def get_cov_thresholds():
    """ Reads coverage thresholds from the config, otherwise sets sensible defaults
    """
    try:
        threshs = config.mosdepth_config['general_stats_coverage']
        assert type(threshs) == list
        assert len(threshs) > 0
        threshs = [int(t) for t in threshs]
        log.debug("Custom coverage thresholds: {}".format(", ".join([str(t) for t in threshs])))
    except (AttributeError, TypeError, AssertionError):
        threshs = [1, 5, 10, 30, 50]
        log.debug("Using default coverage thresholds: {}".format(", ".join([str(t) for t in threshs])))

    try:
        hidden_threshs = config.mosdepth_config['general_stats_coverage_hidden']
        assert type(hidden_threshs) == list
        log.debug("Hiding coverage thresholds: {}".format(", ".join([str(t) for t in hidden_threshs])))
    except (AttributeError, TypeError, KeyError, AssertionError):
        hidden_threshs = [t for t in threshs if t != 30]

    return threshs, hidden_threshs



#!/usr/bin/env python

""" MultiQC module to parse output from VerifyBAMID """

from __future__ import print_function
from collections import OrderedDict
import logging
from multiqc import config
from multiqc.plots import table
from multiqc.modules.base_module import BaseMultiqcModule

# Initialise the logger
log = logging.getLogger(__name__)

class MultiqcModule(BaseMultiqcModule):
    """
     module class, parses stderr logs.
    """

    def __init__(self):
        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='VerifyBAMID', anchor='verifybamid',
            href = 'https://genome.sph.umich.edu/wiki/VerifyBamID',
            info = "detects sample contamination and/or sample swaps.")

        # flag to hide columns if no chip data
        self.hide_chip_columns=True

        # default values for columns
        self.col_config_defaults = {
            'max': 100,
            'min': 0,
            'suffix': '%',
            'format': '{:,.3f}',
            'modify': lambda x:x * 100.0 if x != "NA" else x,
            'scale': 'OrRd'
        }

        # dictionary to hold all data for each sample
        self.verifybamid_data = dict()

        # for each file ending in self.SM
        for f in self.find_log_files('verifybamid/selfsm'):
            # pass the file to function self.parse_selfsm to parse file
            parsed_data = self.parse_selfsm(f)
            # if a result was returned
            if parsed_data is not None:
                # for each sample extracted from the file
                for s_name in parsed_data:
                    # if there are duplicate sample names
                    if s_name in self.verifybamid_data:
                        # write this to log
                        log.debug("Duplicate sample name found! Overwriting: {}".format(s_name))
                    # add the sample as a key to the verifybamid_data dictionary and the dictionary of values as the value
                    self.verifybamid_data[s_name] = parsed_data[s_name]
                # add data source to multiqc_sources.txt
                self.add_data_source(f, s_name)

        # Filter to strip out ignored sample names as per config.yaml
        self.verifybamid_data = self.ignore_samples(self.verifybamid_data)

        if len(self.verifybamid_data) == 0:
            raise UserWarning

        # print number of verifyBAMID reports found and parsed
        log.info("Found {} reports".format(len(self.verifybamid_data)))

        # Write parsed report data to a file
        self.write_data_file(self.verifybamid_data, 'multiqc_verifybamid')

        # add to General Stats Table
        self.verifybamid_general_stats_table()

        # add section with the values from the verify BAM ID output
        self.verifybamid_table()


    def parse_selfsm(self, f):
        """ Go through selfSM file and create a dictionary with the sample name as a key, """
        #create a dictionary to populate from this sample's file
        parsed_data = dict()
        # set a empty variable which denotes if the headers have been read
        headers = None
        # for each line in the file
        for l in f['f'].splitlines():
            # split the line on tab
            s = l.split("\t")
            # if we haven't already read the header line
            if headers is None:
                # assign this list to headers variable
                headers = s
            # for all rows after the first
            else:
                # clean the sample name (first column) and assign to s_name
                s_name = self.clean_s_name(s[0], f['root'])
                # create a dictionary entry with the first column as a key (sample name) and empty dictionary as a value
                parsed_data[s_name] = {}
                # for each item in list of items in the row
                for i, v in enumerate(s):
                    # if it's not the first element (if it's not the name)
                    if i != 0:
                        # see if CHIP is in the column header and the value is not NA
                        if "CHIP" in [headers[i]] and v != "NA":
                            # set hide_chip_columns = False so they are not hidden
                            self.hide_chip_columns=False
                        # try and convert the value into a float
                        try:
                            # and add to the dictionary the key as the corrsponding item from the header and the value from the list
                            parsed_data[s_name][headers[i]] = float(v)
                        #if can't convert to float...
                        except ValueError:
                            # add to the dictionary the key as the corrsponding item from the header and the value from the list
                            parsed_data[s_name][headers[i]] = v

                # Rename some column titles that can be variably named (See issue #1112)
                if 'FREEMIX(alpha)' in parsed_data[s_name]:
                    parsed_data[s_name]['FREEMIX'] = parsed_data[s_name]['FREEMIX(alpha)']

        # else return the dictionary
        return parsed_data

    def verifybamid_general_stats_table(self):
        """ Take the percentage of contamination from all the parsed *.SELFSM files and add it to the basic stats table at the top of the report """

        # create a dictionary to hold the columns to add to the general stats table
        headers = OrderedDict()
        # available columns are:
        #SEQ_ID RG  CHIP_ID #SNPS   #READS  AVG_DP  FREEMIX FREELK1 FREELK0 FREE_RH FREE_RA CHIPMIX CHIPLK1 CHIPLK0 CHIP_RH CHIP_RA DPREF   RDPHET  RDPALT
        #see https://genome.sph.umich.edu/wiki/VerifyBamID#Interpreting_output_files

        # add the CHIPMIX column if we have data
        if not self.hide_chip_columns:
            headers['CHIPMIX'] = dict(self.col_config_defaults, **{
                'title': 'Contamination (S+A)',
                'description': 'VerifyBamID: CHIPMIX -   Sequence+array estimate of contamination (NA if the external genotype is unavailable)'
            })

        # add the FREEMIX column. set the title and description
        headers['FREEMIX'] = dict(self.col_config_defaults, **{
            'title': 'Contamination (S)',
            'description': 'VerifyBamID: FREEMIX -   Sequence-only estimate of contamination.',
        })

        # pass the data dictionary and header dictionary to function to add to table.
        self.general_stats_addcols(self.verifybamid_data, headers)


    def verifybamid_table(self):
        """
        Create a table with all the columns from verify BAM ID
        """

        # create an ordered dictionary to preserve the order of columns
        headers = OrderedDict()
        # add each column and the title and description (taken from verifyBAMID website)
        headers['RG'] = {
            'title': 'Read Group',
            'description': 'ReadGroup ID of sequenced lane.',
            'hidden': all( [ s['RG'] == 'ALL' for s in self.verifybamid_data.values() ] )
        }
        if not self.hide_chip_columns:
            headers['CHIP_ID'] = {
                'title': 'Chip ID',
                'description': 'ReadGroup ID of sequenced lane.'
            }
        headers['#SNPS'] = {
            'title': 'SNPS',
            'description': '# SNPs passing the criteria from the VCF file',
            'format': '{:,.0f}',
            'min': 0,
            'scale': 'BuPu'
        }
        headers['#READS'] = {
            'title': '{} Reads'.format(config.read_count_prefix),
            'description': 'Number of reads loaded from the BAM file ({})'.format(config.read_count_desc),
            'format': '{:,.1f}',
            'modify': lambda x: x * config.read_count_multiplier if x != "NA" else x,
            'shared_key': 'read_count',
            'min': 0,
            'scale': 'GnBu'
        }
        headers['AVG_DP'] = {
            'title': 'Average Depth',
            'description': 'Average sequencing depth at the sites in the VCF file',
            'suffix': ' X',
            'min': 0,
            'scale': 'YlGn'
        }
        # use default columns
        headers['FREEMIX'] = dict(self.col_config_defaults, **{
            'title': 'Contamination (Seq)',
            'description': 'VerifyBamID: FREEMIX -   Sequence-only estimate of contamination.',
        })
        headers['FREELK1'] = {
            'title': 'FREEELK1',
            'format': '{:,.0f}',
            'description': 'Maximum log-likelihood of the sequence reads given estimated contamination under sequence-only method',
            'min': 0,
            'scale': 'RdYlGn'
        }
        headers['FREELK0'] = {
            'title': 'FREELK0',
            'format': '{:,.0f}',
            'description': 'Log-likelihood of the sequence reads given no contamination under sequence-only method',
            'min': 0,
            'scale': 'RdYlGn'
        }
        headers['FREE_RH'] = {
            'title': 'FREE_RH',
            'description': 'Estimated reference bias parameter Pr(refBase|HET) (when --free-refBias or --free-full is used)',
            'hidden': all( [ s['FREE_RH'] == 'NA' for s in self.verifybamid_data.values() ] ),
        }
        headers['FREE_RA'] = {
            'title': 'FREE_RA',
            'description': 'Estimated reference bias parameter Pr(refBase|HOMALT) (when --free-refBias or --free-full is used)',
            'hidden': all( [ s['FREE_RA'] == 'NA' for s in self.verifybamid_data.values() ] ),
        }

        # Only print Chip columns to the report if we have data
        if not self.hide_chip_columns:
            headers['CHIPMIX'] = dict(self.col_config_defaults, **{
                'title': 'Contamination S+A',
                'description': 'VerifyBamID: CHIPMIX -   Sequence+array estimate of contamination (NA if the external genotype is unavailable)'
            })
            headers['CHIPLK1'] = {
                'title': 'CHIPLK1',
                'description': 'Maximum log-likelihood of the sequence reads given estimated contamination under sequence+array method (NA if the external genotypes are unavailable)'
            }
            headers['CHIPLK0'] = {
                'title': 'CHIPLK0',
                'description': ' Log-likelihood of the sequence reads given no contamination under sequence+array method (NA if the external genotypes are unavailable)'
                }
            headers['CHIP_RH'] = {
                'title': 'CHIP_RH',
                'description': 'Estimated reference bias parameter Pr(refBase|HET) (when --chip-refBias or --chip-full is used)'
            }
            headers['CHIP_RA'] = {
                'title': 'CHIP_RA',
                'description': 'Estimated reference bias parameter Pr(refBase|HOMALT) (when --chip-refBias or --chip-full is used)'
            }

        headers['DPREF'] = {
            'title': 'DPREF',
            'description': 'Depth (Coverage) of HomRef site (based on the genotypes of (SELF_SM/BEST_SM), passing mapQ, baseQual, maxDepth thresholds.',
            'hidden': all( [ s['DPREF'] == 'NA' for s in self.verifybamid_data.values() ] ),
        }
        headers['RDPHET'] = {
            'title': 'RDPHET',
            'description': 'DPHET/DPREF, Relative depth to HomRef site at Heterozygous site.',
            'hidden': all( [ s['RDPHET'] == 'NA' for s in self.verifybamid_data.values() ] ),
        }
        headers['RDPALT'] = {
            'title': 'RDPALT',
            'description': 'DPHET/DPREF, Relative depth to HomRef site at HomAlt site.',
            'hidden': all( [ s['RDPALT'] == 'NA' for s in self.verifybamid_data.values() ] ),
        }

        tconfig = {
            'namespace': 'VerifyBAMID',
            'id': 'verifybamid-results',
        }

        # send the plot to add section function with data dict and headers
        self.add_section (
            anchor = 'verifybamid-table',
            description = 'The following values provide estimates of sample contamination. Click help for more information.',
            helptext = '''
            **Please note that `FREEMIX` is named _Contamination (Seq)_ and `CHIPMIX`
            is named _Contamination (S+A)_ in this MultiQC report.**

            VerifyBamID provides a series of information that is informative to determine
            whether the sample is possibly contaminated or swapped, but there is no single
            criteria that works for every circumstances. There are a few unmodeled factor
            in the estimation of `[SELF-IBD]/[BEST-IBD]` and `[%MIX]`, so please note that the
            MLE estimation may not always exactly match to the true amount of contamination.
            Here we provide a guideline to flag potentially contaminated/swapped samples:

            * Each sample or lane can be checked in this way.
              When `[CHIPMIX] >> 0.02` and/or `[FREEMIX] >> 0.02`, meaning 2% or more of
              non-reference bases are observed in reference sites, we recommend to examine
              the data more carefully for the possibility of contamination.
            * We recommend to check each lane for the possibility of sample swaps.
              When `[CHIPMIX] ~ 1` AND `[FREEMIX] ~ 0`, then it is possible that the sample
              is swapped with another sample. When `[CHIPMIX] ~ 0` in `.bestSM` file,
              `[CHIP_ID]` might be actually the swapped sample. Otherwise, the swapped
              sample may not exist in the genotype data you have compared.
            * When genotype data is not available but allele-frequency-based estimates of
              `[FREEMIX] >= 0.03` and `[FREELK1]-[FREELK0]` is large, then it is possible
              that the sample is contaminated with other sample. We recommend to use
              per-sample data rather than per-lane data for checking this for low coverage
              data, because the inference will be more confident when there are large number
              of bases with depth 2 or higher.

            _Copied from the [VerifyBAMID documentation](https://genome.sph.umich.edu/wiki/VerifyBamID) - see the link for more details._
            ''',
            plot = table.plot(self.verifybamid_data, headers, tconfig)
        )



#!/usr/bin/env python
""" MultiQC module to parse output from HOMER """
from __future__ import print_function
from collections import OrderedDict
import logging

from multiqc.modules.base_module import BaseMultiqcModule

# Import the HOMER submodules
from .findpeaks import FindPeaksReportMixin
from .tagdirectory import TagDirReportMixin

# Initialise the logger
log = logging.getLogger(__name__)


class MultiqcModule(BaseMultiqcModule, FindPeaksReportMixin, TagDirReportMixin):
    """ HOMER has a number of different commands and outputs.
    This MultiQC module supports some but not all. The code for
    each script is split into its own file and adds a section to
    the module output if logs are found. """

    def __init__(self):

        # Initialise the parent object
        super(MultiqcModule, self).__init__(name='HOMER', anchor='homer',
            href='http://homer.ucsd.edu/homer/',
            info="is a suite of tools for Motif Discovery and next-gen sequencing analysis.")


        # Set up class objects to hold parsed data
        self.general_stats_headers = OrderedDict()
        self.general_stats_data = dict()
        n = dict()

         # Set up data structures
        self.tagdir_data = {
            'GCcontent': {},
            'restriction': {},
            'restriction_norm': {},
            'length': {},
            'taginfo_total': {},
            'taginfo_total_norm': {},
            'taginfo_uniq': {},
            'taginfo_uniq_norm': {},
            'FreqDistribution' : {},
            'header' : {},
            'interChr': {}

        }
        # Call submodule functions


        n['findpeaks'] = self.parse_homer_findpeaks()
        if n['findpeaks'] > 0:
            log.info("Found {} findPeaks reports".format(n['findpeaks']))


        n['tagDir'] = self.homer_tagdirectory()
        if n['tagDir'] > 0:
            log.info("Found {} tagDir reports".format(n['tagDir']))

        # Exit if we didn't find anything
        if sum(n.values()) == 0:
            raise UserWarning

        # Add to the General Stats table (has to be called once per MultiQC module)
        self.general_stats_addcols(self.general_stats_data, self.general_stats_headers)
